import numpy as np
from collections import OrderedDict
import copy

try:
    import emcee
except ImportError:
    print("You will need to install 'emcee' if you wish to train your own classifier on new data.")


def fit_early_lightcurve(outlc, earlytime=10):
    """
    Return tsquarize fit to early light curve
    """

    def fit_all_pb_light_curves(params, times, fluxes, fluxerrs):
        print(params)
        t0 = params[0]
        pars = params[1:]

        chi2 = 0
        for i, pb in enumerate(times):
            a, c = pars[i * 2:i * 2 + 2]

            model = np.heaviside((times[pb] - t0), 1) * (a * (times[pb] - t0) ** 2) + c
            chi2 += sum((fluxes[pb] - model) ** 2 / fluxerrs[pb] ** 2)

        print(chi2)
        return chi2

    def fit_func(t, a, c, t0):
        return np.heaviside((t - t0), 1) * (a * (t - t0) ** 2) + c

    copy.deepcopy(outlc)
    times = OrderedDict()
    fluxes = OrderedDict()
    fluxerrs = OrderedDict()
    passbands = np.unique(outlc['passband'].data)
    for i, pb in enumerate(passbands):
        pbmask = outlc['passband'] == pb
        ttime = outlc[pbmask]['time'].data
        tflux = outlc[pbmask]['flux'].data
        tfluxerr = outlc[pbmask]['fluxErr'].data
        tphotflag = outlc[pbmask]['photflag'].data

        minfluxpb = tflux.min()
        maxfluxpb = tflux.max()
        norm = maxfluxpb - minfluxpb

        tfluxrenorm = (tflux - minfluxpb) / norm
        tfluxerrrenorm = tfluxerr / norm

        if len(ttime) <= 1 or not np.any(ttime < 0):
            tfluxrenorm = tfluxrenorm - min(tfluxrenorm)
        else:
            tfluxrenorm = tfluxrenorm - np.median(tfluxrenorm[ttime < 0])
        # mask = ttime > -30
        # tfluxrenorm = tfluxrenorm[mask]
        # ttime = ttime[mask]
        # tfluxerrrenorm = tfluxerrrenorm[mask]

        earlymask = ttime <= earlytime
        times[pb] = ttime[earlymask]
        fluxes[pb] = tfluxrenorm[earlymask]
        fluxerrs[pb] = tfluxerrrenorm[earlymask]

    remove_pbs = []
    x0 = [-12]
    bounds = [(-25, 0)]
    for pb in fluxes:
        if fluxes[pb].size == 0:
            remove_pbs.append(pb)
        else:
            x0 += [np.mean(fluxes[pb]), np.median(fluxes[pb])]
            bounds += [(0, max(fluxes[pb])), (min(fluxes[pb]), max(fluxes[pb]))]

    for pb in remove_pbs:
        del times[pb]
        del fluxes[pb]
        del fluxerrs[pb]

    # optimise_result = minimize(fit_all_pb_light_curves, x0=x0, args=(times, fluxes, fluxerrs), bounds=bounds)
    # t0 = optimise_result.x[0]
    # bestpars = optimise_result.x[1:]
    # best = {pb: np.append(bestpars[i*2:i*2+2], t0) for i, pb in enumerate(times)}

    ndim = len(x0)

    best = emcee_fit_all_pb_lightcurves(times, fluxes, fluxerrs, ndim, np.array(x0), bounds)
    # best, covariance = curve_fit(fit_func, time, flux, sigma=fluxerr, p0=[max(flux), min(flux)])

    # best = fit_all_pb_lightcurves(time, flux, fluxerr)
    print('best', best)

    return fit_func, best


def lnlike(params, times, fluxes, fluxerrs):
    # print(params)
    t0 = params[0]
    pars = params[1:]

    chi2 = 0
    for i, pb in enumerate(times):
        a, c = pars[i * 2:i * 2 + 2]

        model = np.heaviside((times[pb] - t0), 1) * (a * (times[pb] - t0) ** 2) + c
        chi2 += sum((fluxes[pb] - model) ** 2 / fluxerrs[pb] ** 2)

        # print(pb, a, c)

    # print('chi2', chi2, params)
    return np.exp(-0.5*chi2)


def lnprior(params):
    # print('params', params)
    t0 = params[0]
    pars = params[1:]

    for par in pars:
        if par > 1e3 or par < -1e3:
            return -np.inf

    if -35 < t0 < 0:
        return 0.0

    return -np.inf


def lnprob(params, t, flux, fluxerr):
    lp = lnprior(params)
    if not np.isfinite(lp):
        return -np.inf
    return lp + lnlike(params, t, flux, fluxerr)


def emcee_fit_all_pb_lightcurves(times, fluxes, fluxerrs, ndim, x0=None, bounds=None):
    nwalkers = 200
    nsteps = 700
    burn = 5
    pos = np.array([x0 + (([3] + len(times.keys()) * [0.1, 0.05]) * np.random.randn(ndim)) for i in range(nwalkers)])
    # print(pos[:,0])

    # print("DATA IS TO FOLLOW")
    # print("times")
    # print(times)
    # print("fluxes")
    # print(fluxes
    # print("fluxerrs")
    # print(fluxerrs)

    print("running mcmc...")

    # Ensure intial params within parameter bounds
    params = OrderedDict()
    params['t0'] = {'bounds': bounds[0], 'value': x0[0], 'scale': 1}
    i = 1
    for pb in times.keys():
        for name in ['a', 'c']:
            params[pb + ': ' + name] = {'bounds': bounds[i], 'value': x0[i], 'scale': 3}
            i += 1
    for i, name in enumerate(params.keys()):
        # import pdb; pdb.set_trace()
        lb, ub = params[name]['bounds']
        p0 = params[name]['value']
        std = params[name]['scale']
        # take a 5 sigma range
        lr, ur = (p0 - 5. * std, p0 + 5. * std)
        ll = max(lb, lr, 0.)
        ul = min(ub, ur)
        ind = np.where((pos[:, i] <= ll) | (pos[:, i] >= ul))
        nreplace = len(pos[:, i][ind])
        pos[:, i][ind] = np.random.rand(nreplace) * (ul - ll) + ll
    # print('new', np.array(pos)[:,0])
    # print(pos)

    sampler = emcee.EnsembleSampler(nwalkers, ndim, lnprob, args=(times, fluxes, fluxerrs))

    # FInd parameters of lowest chi2
    pos, prob, state = sampler.run_mcmc(pos, 1)
    opos, oprob, orstate = [], [], []
    for pos, prob, rstate in sampler.sample(pos, prob, state, iterations=nsteps):
        opos.append(pos.copy())
        oprob.append(prob.copy())
    pos = np.array(opos)
    prob = np.array(oprob)
    nstep, nwalk = np.unravel_index(prob.argmax(), prob.shape)
    bestpars = pos[nstep, nwalk]
    posterior = prob

    print("best", bestpars)
    # import pylab
    # for j in range(nwalkers):
    #     pylab.plot(posterior[:, j])
    # for i in range(len(params)):
    #     pylab.figure()
    #     pylab.title(list(params.keys())[i])
    #     for j in range(nwalkers):
    #         pylab.plot(pos[:, j, i])

    samples = sampler.chain[:, burn:, :].reshape((-1, ndim))
    # samples[:, 2] = np.exp(samples[:, 2])
    # bestpars2 = list(map(lambda v: (v[0]), zip(*np.percentile(samples, [50], axis=0))))
    # print('b2', bestpars2)

    print(bestpars)
    t0 = bestpars[0]
    bestpars = bestpars[1:]

    # for i, pb in enumerate(times):
    #     if len(times[pb][times[pb] > t0]) == 0:
    #         bestpars[i * 2] = 0  # set `a' to zero if no data points above t0

    best = {pb: np.append(bestpars[i * 2:i * 2 + 2], t0) for i, pb in enumerate(times)}

    # to_delete = []
    # for i, name in enumerate(params):
    #     if not np.any(samples[:, i]):  # all zeros for parameter then delete column
    #         to_delete.append((i, name))
    # for i, name in to_delete:
    #     samples = np.delete(samples, i, axis=1)
    #     print(name)
    #     del params[name]
    #
    # np.save('samples', samples[::100])

    return best



import os
import h5py
from sklearn.model_selection import train_test_split
from tensorflow.keras.utils import to_categorical
import pickle
import multiprocessing as mp
import numpy as np
import pandas as pd
import itertools
from scipy.interpolate import interp1d

from astrorapid import helpers
from astrorapid.get_training_data import get_data

# fix random seed for reproducibility
np.random.seed(42)


class PrepareArrays(object):
    def __init__(self, passbands=('g', 'r'), contextual_info=('redshift',)):
        self.passbands = passbands
        self.contextual_info = contextual_info
        self.nobs = 50  # 100
        self.npassbands = len(passbands)
        self.nfeatures = self.npassbands + len(self.contextual_info)  # 4 + len(self.contextual_info)  #
        self.timestep = 3.0
        self.mintime = -70
        self.maxtime = 80

    def make_cuts(self, data, i, deleterows, b, redshift=None, class_num=None, bcut=True, zcut=0.5, variables_cut=True,
                  pre_trigger=True):
        deleted = False
        try:
            time = data[data['passband']=='r']['time'][0:self.nobs].data
        except KeyError:
            print("No r band data. passbands")
            deleterows.append(i)
            deleted = True
            return deleterows, deleted

        if len(data) < 4:
            print("Less than 4 epochs. nobs = {}".format(len(data)))
            deleterows.append(i)
            deleted = True
        elif pre_trigger and len(time[time < 0]) < 3:
            print("Less than 3 points in the r band pre trigger", len(time[time < 0]))
            deleterows.append(i)
            deleted = True
        elif bcut and abs(b) < 15:
            print("In galactic plane. b = {}".format(b))
            deleterows.append(i)
            deleted = True
        elif zcut is not None and redshift is not None and (redshift > self.zcut or redshift == 0):
            print("Redshift cut. z = {}".format(redshift))
            deleterows.append(i)
            deleted = True
        elif class_num is not None and variables_cut is True and class_num in [70, 80, 81, 83, 84, 90, 91, 92, 93]:
            print("Not including variable models", class_num)
            deleterows.append(i)
            deleted = True
        elif class_num in [50, 61, 62]:
            print("Deleting unused kilonova class and PISN and ILOT")
            deleterows.append(i)
            deleted = True

        return deleterows, deleted

    def get_min_max_time(self, data):
        # Get min and max times for tinterp
        mintimes = []
        maxtimes = []
        for j, pb in enumerate(self.passbands):
            pbmask = data['passband']==pb
            time = data[pbmask]['time'][0:self.nobs].data
            try:
                mintimes.append(time.min())
                maxtimes.append(time.max())
            except ValueError:
                print("No data for passband: ", pb)
        mintime = min(mintimes)
        maxtime = max(maxtimes) + self.timestep

        return mintime, maxtime

    def get_t_interp(self, data):
        mintime, maxtime = self.get_min_max_time(data)

        tinterp = np.arange(mintime, maxtime, step=self.timestep)
        len_t = len(tinterp)
        if len_t > self.nobs:
            tinterp = tinterp[(tinterp >= self.mintime)]
            len_t = len(tinterp)
            if len_t > self.nobs:
                tinterp = tinterp[:-(len_t - self.nobs)]
                len_t = len(tinterp)
        return tinterp, len_t

    def update_X(self, X, i, data, tinterp, len_t, objid, contextual_info, meta_data):
        for j, pb in enumerate(self.passbands):
            # Drop infinite or nan values in any row
            data.remove_rows(np.where(~np.isfinite(data['time']))[0])
            data.remove_rows(np.where(~np.isfinite(data['flux']))[0])
            data.remove_rows(np.where(~np.isfinite(data['fluxErr']))[0])

            # Get data
            pbmask = data['passband']==pb
            time = data[pbmask]['time'][0:self.nobs].data
            flux = data[pbmask]['flux'][0:self.nobs].data
            fluxerr = data[pbmask]['fluxErr'][0:self.nobs].data
            photflag = data[pbmask]['photflag'][0:self.nobs].data

            # Mask out times outside of mintime and maxtime
            timemask = (time > self.mintime) & (time < self.maxtime)
            time = time[timemask]
            flux = flux[timemask]
            fluxerr = fluxerr[timemask]
            photflag = photflag[timemask]

            n = len(flux)  # Get vector length (could be less than nobs)

            if n > 1:
                # if flux[-1] > flux[-2]:  # If last values are increasing, then set fill_values to zero
                #     f = interp1d(time, flux, kind='linear', bounds_error=False, fill_value=0.)
                # else:
                #     f = interp1d(time, flux, kind='linear', bounds_error=False,
                #                  fill_value='extrapolate')  # extrapolate until all passbands finished.
                f = interp1d(time, flux, kind='linear', bounds_error=False, fill_value=0.)

                fluxinterp = f(tinterp)
                fluxinterp = np.nan_to_num(fluxinterp)
                fluxinterp = fluxinterp.clip(min=0)
                fluxerrinterp = np.zeros(len_t)

                for interp_idx, fluxinterp_val in enumerate(fluxinterp):
                    if fluxinterp_val == 0.:
                        fluxerrinterp[interp_idx] = 0
                    else:
                        nearest_idx = helpers.find_nearest(time, tinterp[interp_idx])
                        fluxerrinterp[interp_idx] = fluxerr[nearest_idx]

                X[i][j][0:len_t] = fluxinterp
                # X[i][j * 2 + 1][0:len_t] = fluxerrinterp

        # Add contextual information
        for jj, c_info in enumerate(contextual_info, 1):
            X[i][j + jj][0:len_t] = meta_data[c_info] * np.ones(len_t)

        return X


class PrepareInputArrays(PrepareArrays):
    def __init__(self, passbands=('g', 'r'), contextual_info=('redshift',), bcut=True, zcut=None):
        PrepareArrays.__init__(self, passbands, contextual_info)
        self.passbands = passbands
        self.contextual_info = contextual_info
        self.bcut = bcut
        self.zcut = zcut

    def prepare_input_arrays(self, lightcurves):
        nobjects = len(lightcurves)

        X = np.zeros(shape=(nobjects, self.nfeatures, self.nobs))
        timesX = np.zeros(shape=(nobjects, self.nobs))
        objids_list = []
        orig_lc = []
        deleterows = []
        trigger_mjds = []

        for i, (objid, data) in enumerate(lightcurves.items()):
            print("Preparing light curve {} of {}".format(i, nobjects))

            redshift = data.meta['redshift']
            b = data.meta['b']
            trigger_mjd = data.meta['trigger_mjd']

            # Make cuts
            deleterows, deleted = self.make_cuts(data, i, deleterows, b, redshift, class_num=None, bcut=self.bcut,
                                                 zcut=self.zcut, pre_trigger=False)
            if deleted:
                continue

            tinterp, len_t = self.get_t_interp(data)
            timesX[i][0:len_t] = tinterp
            orig_lc.append(data)
            objids_list.append(objid)
            trigger_mjds.append(trigger_mjd)
            X = self.update_X(X, i, data, tinterp, len_t, objid, self.contextual_info, data.meta)

        deleterows = np.array(deleterows)
        X = np.delete(X, deleterows, axis=0)
        timesX = np.delete(timesX, deleterows, axis=0)

        # Correct shape for keras is (N_objects, N_timesteps, N_passbands) (where N_timesteps is lookback time)
        X = X.swapaxes(2, 1)

        return X, orig_lc, timesX, objids_list, trigger_mjds


class PrepareTrainingSetArrays(PrepareArrays):
    def __init__(self, passbands=('g', 'r'), contextual_info=('redshift',), reread=False, aggregate_classes=False, bcut=True,
                 zcut=None, variablescut=False, nchunks=10000, training_set_dir='data/training_set_files',
                 data_dir='data/ZTF_20190512/', save_dir='data/saved_light_curves/', get_data_func=None):
        PrepareArrays.__init__(self, passbands, contextual_info)
        self.passbands = passbands
        self.contextual_info = contextual_info
        self.reread = reread
        self.aggregate_classes = aggregate_classes
        self.bcut = bcut
        self.zcut = zcut
        self.variablescut = variablescut
        self.nchunks = nchunks
        self.agg_map = helpers.aggregate_sntypes()
        self.training_set_dir = training_set_dir
        self.data_dir = data_dir
        self.save_dir = save_dir
        self.light_curves = {}
        self.get_data_func = get_data_func
        if 'redshift' in contextual_info:
            self.known_redshift = True
        else:
            self.known_redshift = False
        if not os.path.exists(self.training_set_dir):
            os.makedirs(self.training_set_dir)

    def get_light_curves(self, class_nums=(1,), nprocesses=1):
        light_curves = {}

        for class_num in class_nums:
            lcs = get_data(self.get_data_func, class_num, self.data_dir, self.save_dir, self.passbands,
                           self.known_redshift, nprocesses, self.reread)
            light_curves.update(lcs)

        return light_curves

    def prepare_training_set_arrays(self, otherchange='', class_nums=(1,), nprocesses=1, train_size=0.6):
        savepath = os.path.join(self.training_set_dir,
                                "X_{}ag{}_ci{}_z{}_b{}_var{}.npy".format(otherchange, self.aggregate_classes,
                                                                              self.contextual_info, self.zcut,
                                                                              self.bcut, self.variablescut))
        print(savepath)
        if self.reread is True or not os.path.isfile(savepath):
            self.light_curves = self.get_light_curves(class_nums, nprocesses)

            objids = list(set(self.light_curves.keys()))
            nobjects = len(objids)

            # Store data labels (y) and 'r' band data (X). Use memory mapping because input file is very large.
            labels = np.zeros(shape=nobjects, dtype=np.uint16)
            y = np.zeros(shape=(nobjects, self.nobs), dtype=np.uint16)
            X = np.memmap(os.path.join(self.training_set_dir, 'X_lc_data.dat'), dtype=np.float32, mode='w+',
                          shape=(nobjects, self.nfeatures, self.nobs))  # 4+len(self.contextual_info), 100))
            X[:] = np.zeros(shape=(nobjects, self.nfeatures, self.nobs))
            timesX = np.zeros(shape=(nobjects, self.nobs))
            objids_list = []
            orig_lc = []

            # Chunk before multiprocessing
            multi_objids = np.array_split(objids, self.nchunks)

            # Store light curves into X (fluxes) and y (labels)
            pool = mp.Pool(nprocesses)
            results = pool.map_async(self.multi_read_obj, multi_objids) ##
            pool.close()
            pool.join()

            outputs = results.get()

            # outputs = self.multi_read_obj_nonuniformtime(objids[0:100])

            sum_deleterows = 0
            startidx = 0
            num_outputs = len(outputs)
            print('combining results...')
            for i, output in enumerate(outputs):
                labels_part, y_part, X_part, timesX_part, objids_list_part, orig_lc_part, num_deleterows_part, num_objects_part = output
                endidx = startidx + num_objects_part
                labels[startidx:endidx] = labels_part
                y[startidx:endidx] = y_part
                X[startidx:endidx] = X_part
                timesX[startidx:endidx] = timesX_part
                objids_list.extend(objids_list_part)
                orig_lc.extend(orig_lc_part)
                startidx += num_objects_part
                sum_deleterows += num_deleterows_part

            deleterows = np.array(np.arange(nobjects - sum_deleterows, nobjects))
            X = np.delete(X, deleterows, axis=0)
            y = np.delete(y, deleterows, axis=0)
            labels = np.delete(labels, deleterows, axis=0)
            timesX = np.delete(timesX, deleterows, axis=0)

            np.save(os.path.join(self.training_set_dir,
                                 "X_{}ag{}_ci{}_z{}_b{}_var{}.npy".format(otherchange, self.aggregate_classes,
                                                                               self.contextual_info,
                                                                               self.zcut, self.bcut,
                                                                               self.variablescut)), X)
            np.save(os.path.join(self.training_set_dir,
                                 "y_{}ag{}_ci{}_z{}_b{}_var{}.npy".format(otherchange, self.aggregate_classes,
                                                                               self.contextual_info,
                                                                               self.zcut, self.bcut,
                                                                               self.variablescut)), y)
            np.save(os.path.join(self.training_set_dir,
                                 "labels_{}ag{}_ci{}_z{}_b{}_var{}.npy".format(otherchange, self.aggregate_classes,
                                                                                    self.contextual_info,
                                                                                    self.zcut,
                                                                                    self.bcut, self.variablescut)),
                    labels)
            np.save(os.path.join(self.training_set_dir, "tinterp_{}ag{}_ci{}_z{}_b{}_var{}.npy".format(otherchange,
                                                                                                            self.aggregate_classes,
                                                                                                            self.contextual_info,

                                                                                                            self.zcut,
                                                                                                            self.bcut,
                                                                                                            self.variablescut)),
                    timesX)
            np.save(os.path.join(self.training_set_dir,
                                 "objids_{}ag{}_ci{}_z{}_b{}_var{}.npy".format(otherchange, self.aggregate_classes,
                                                                                    self.contextual_info,
                                                                                    self.zcut,
                                                                                    self.bcut, self.variablescut)),
                    objids_list)
            with open(os.path.join(self.training_set_dir,
                                   "origlc_{}ag{}_ci{}_z{}_b{}_var{}.npy".format(otherchange,
                                                                                      self.aggregate_classes,
                                                                                      self.contextual_info,
                                                                                      self.zcut,
                                                                                      self.bcut, self.variablescut)),
                      'wb') as f:
                pickle.dump(orig_lc, f)

        else:
            X = np.load(os.path.join(self.training_set_dir,
                                     "X_{}ag{}_ci{}_z{}_b{}_var{}.npy".format(otherchange, self.aggregate_classes,
                                                                                   self.contextual_info,
                                                                                   self.zcut, self.bcut,
                                                                                   self.variablescut)), mmap_mode='r')
            y = np.load(os.path.join(self.training_set_dir,
                                     "y_{}ag{}_ci{}_z{}_b{}_var{}.npy".format(otherchange, self.aggregate_classes,
                                                                                   self.contextual_info,
                                                                                   self.zcut, self.bcut,
                                                                                   self.variablescut)))
            labels = np.load(os.path.join(self.training_set_dir,
                                          "labels_{}ag{}_ci{}_z{}_b{}_var{}.npy".format(otherchange,
                                                                                             self.aggregate_classes,
                                                                                             self.contextual_info,
                                                                                             self.zcut,
                                                                                             self.bcut,
                                                                                             self.variablescut)))
            timesX = np.load(os.path.join(self.training_set_dir,
                                          "tinterp_{}ag{}_ci{}_z{}_b{}_var{}.npy".format(otherchange,
                                                                                              self.aggregate_classes,
                                                                                              self.contextual_info,
                                                                                              self.zcut,
                                                                                              self.bcut,
                                                                                              self.variablescut)))
            objids_list = np.load(os.path.join(self.training_set_dir,
                                               "objids_{}ag{}_ci{}_z{}_b{}_var{}.npy".format(otherchange,
                                                                                                  self.aggregate_classes,
                                                                                                  self.contextual_info,
                                                                                                  self.zcut,
                                                                                                  self.bcut,
                                                                                                  self.variablescut)))
            with open(os.path.join(self.training_set_dir,
                                   "origlc_{}ag{}_ci{}_z{}_b{}_var{}.npy".format(otherchange,
                                                                                      self.aggregate_classes,
                                                                                      self.contextual_info,
                                                                                      self.zcut,
                                                                                      self.bcut, self.variablescut)),
                      'rb') as f:
                orig_lc = pickle.load(f)

        classes = sorted(list(set(labels)))
        sntypes_map = helpers.get_sntypes()
        class_names = [sntypes_map[class_num] for class_num in classes]

        # Count nobjects per class
        for c in classes:
            nobs = len(X[labels == c])
            print(c, nobs)

        # Use class numbers 1,2,3... instead of 1, 3, 13 etc.
        y_indexes = np.copy(y)
        for i, c in enumerate(classes):
            y_indexes[y == c] = i + 1
        y = y_indexes

        y = to_categorical(y)

        # Correct shape for keras is (N_objects, N_timesteps, N_passbands) (where N_timesteps is lookback time)
        X = X.swapaxes(2, 1)

        # #NORMALISE
        # X = X.copy()
        # for i in range(len(X)):
        #     for pbidx in range(2):
        #         minX = X[i, :, pbidx].min(axis=0)
        #         maxX = X[i, :, pbidx].max(axis=0)
        #         X[i, :, pbidx] = (X[i, :, pbidx] - minX) / (maxX - minX)
        #         # if (maxX - minX) != 0:
        #         #     mask.append(i)
        #         #     break
        # finitemask = ~np.any(np.any(~np.isfinite(X), axis=1), axis=1)
        # X = X[finitemask]
        # y = y[finitemask]
        # timesX = timesX[finitemask]
        # objids_list = objids_list[finitemask]
        # orig_lc = list(itertools.compress(orig_lc, finitemask))
        # labels = labels[finitemask]

        newX = np.zeros(X.shape)
        newy = np.zeros(y.shape)
        lenX = len(X)
        for i in range(lenX):
            if i % 1000 == 0:
                print(f"new {i} of {lenX}")
            mask = timesX[i] > 0
            nmask = sum(mask)
            newX[i][:nmask] = X[i][mask]
            newy[i][:nmask] = y[i][mask]

        print("Concatenating")
        X = np.concatenate((X, newX))
        y = np.concatenate((y, newy))
        labels = np.concatenate((labels, labels))
        timesX = np.concatenate((timesX, timesX))
        orig_lc = orig_lc * 2
        objids_list = np.concatenate((objids_list, objids_list))
        print("Shuffling")
        from sklearn.utils import shuffle
        X, y, labels, timesX, orig_lc, objids_list = shuffle(X, y, labels, timesX, orig_lc, objids_list)
        print("Done shuffling")



        X_train, X_test, y_train, y_test, labels_train, labels_test, timesX_train, timesX_test, orig_lc_train, \
        orig_lc_test, objids_train, objids_test = train_test_split(
            X, y, labels, timesX, orig_lc, objids_list, train_size=train_size, shuffle=False, random_state=42)

        counts = np.unique(labels_train, return_counts=True)[-1]
        class_weights = max(counts) / counts
        class_weights = dict(zip(range(len(counts)), class_weights))
        print("Class weights:", class_weights)

        # Sample weights
        l_train_indexes = np.copy(labels_train)
        for i, c in enumerate(classes):
            l_train_indexes[l_train_indexes == c] = i
        sample_weights = np.zeros(len(l_train_indexes))
        for key, val in class_weights.items():
            sample_weights[l_train_indexes == key] = val

        return X_train, X_test, y_train, y_test, labels_train, labels_test, class_names, class_weights, \
               sample_weights, timesX_train, timesX_test, orig_lc_train, orig_lc_test, objids_train, objids_test

    def multi_read_obj(self, objids):
        nobjects = len(objids)

        labels = np.zeros(shape=nobjects, dtype=np.uint16)
        y = np.zeros(shape=(nobjects, self.nobs), dtype=np.uint16)
        X = np.zeros(shape=(nobjects, self.nfeatures, self.nobs))
        timesX = np.zeros(shape=(nobjects, self.nobs))
        objids_list = []
        orig_lc = []
        deleterows = []

        for i, objid in enumerate(objids):
            print("Preparing {} light curve {} of {}".format(objid, i, nobjects))

            # Get aggregate model
            model, snid = objid.astype(str).split('_')
            if self.aggregate_classes:
                model = self.agg_map[int(model)]
            class_num = int(model)

            # Get data for each object
            data = self.light_curves[objid]

            redshift = data.meta['redshift']
            b = data.meta['b']
            t0 = data.meta['t0']

            # Make cuts
            deleterows, deleted = self.make_cuts(data, i, deleterows, b, redshift, class_num=model, bcut=self.bcut,
                                                 zcut=self.zcut, variables_cut=self.variablescut, pre_trigger=False)
            if deleted:
                continue

            tinterp, len_t = self.get_t_interp(data)
            timesX[i][0:len_t] = tinterp
            orig_lc.append(data)
            objids_list.append(objid)
            X = self.update_X(X, i, data, tinterp, len_t, objid, self.contextual_info, data.meta)

            activeindexes = (tinterp > t0)
            labels[i] = class_num
            y[i][0:len_t] = class_num * activeindexes

        deleterows = np.array(deleterows)
        X = np.delete(X, deleterows, axis=0)
        y = np.delete(y, deleterows, axis=0)
        labels = np.delete(labels, deleterows, axis=0)
        timesX = np.delete(timesX, deleterows, axis=0)
        count_deleterows = len(deleterows)
        num_objects = X.shape[0]

        return labels, y, X, timesX, objids_list, orig_lc, count_deleterows, num_objects




import os
import numbers
import numpy as np
from collections import OrderedDict
from tensorflow.keras.models import load_model
from pkg_resources import resource_filename
from distutils.spawn import find_executable

from astrorapid.process_light_curves import read_multiple_light_curves
from astrorapid.prepare_arrays import PrepareInputArrays

try:
    import matplotlib.pyplot as plt
    from matplotlib.ticker import MaxNLocator
    import matplotlib
    import matplotlib.animation as animation

    # Check if latex is installed
    if find_executable('latex'):
        plt.rcParams['text.usetex'] = True
    plt.rcParams['font.serif'] = ['Computer Modern Roman'] + plt.rcParams['font.serif']

except ImportError:
    print("Warning: You will need to install 'matplotlib' if you wish to plot the classifications.")


CLASS_NAMES = ['Pre-explosion', 'SNIa-norm', 'SNIbc', 'SNII', 'SNIa-91bg', 'SNIa-x', 'point-Ia', 'Kilonova', 'SLSN-I',
               'PISN', 'ILOT', 'CART', 'TDE']#, 'AGN']
CLASS_COLOR = {'Pre-explosion': 'grey', 'SNIa-norm': 'tab:green', 'SNIbc': 'tab:orange', 'SNII': 'tab:blue',
               'SNIa-91bg': 'tab:red', 'SNIa-x': 'tab:purple', 'point-Ia': 'tab:brown', 'Kilonova': '#aaffc3',
               'SLSN-I': 'tab:olive', 'PISN': 'tab:cyan', 'ILOT': '#FF1493', 'CART': 'navy', 'TDE': 'tab:pink',
               'AGN': 'bisque'}
PB_COLOR = {'u': 'tab:blue', 'g': 'tab:blue', 'r': 'tab:orange', 'i': 'm', 'z': 'k', 'Y': 'y'}
PB_MARKER = {'g': 'o', 'r': 's'}
PB_ALPHA = {'g': 0.3, 'r': 1.}
SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))


class Classify(object):
    def __init__(self, known_redshift=True, model_filepath='', passbands=('g', 'r'),
                 bcut=False, zcut=None, graph=None, model=None):
        """ Takes a list of photometric information and classifies light curves as a function of time

        Parameters
        ----------
        known_redshift : bool
            Different model to be used if redshift is not known.
        model_filepath : str
            Optional argument. The model is taken from the pre-trained model ZTF model if not specified.
        passbands : tuple
            Optional argument. A tuple listing each passband. E.g. ('g', 'r').
        bcut : bool
            Cut on galactic latitude.
            Do not set unless you know what you are doing.
        zcut : float or None
            Remove redshifts above this value.
            Do not set unless you know what you are doing.
        graph : tensorflow graph
            Do not set unless you know what you are doing.
            If you are running astrorapid in multiple threads you may need to predefine this
            This would have been created with Tensorflow i.e. graph = tf.get_default_graph()
        model : tensorflow model
            Do not set unless you know what you are doing.
            If you are running astrorapid in multiple threads you may need to predefine this
            This would have been created with keras' load_model function e.g. model = load_model('keras_model.hdf5')

        """
        self.known_redshift = known_redshift
        self.passbands = passbands
        self.bcut = bcut
        self.zcut = zcut
        self.class_names = CLASS_NAMES

        if self.known_redshift:
            self.contextual_info = ('redshift',)
            filename = '/Users/danmuth/OneDrive - University of Cambridge/PycharmProjects/astrorapid/training_set_files/Figures/classify/ZTF__noAGN_batch500_unnormalised_epochs50_agTrue_ci(0,)_zcut0.5_bcutTrue_varcutTrue/keras_model.hdf5' #'/Users/danmuth/OneDrive - University of Cambridge/PycharmProjects/astrorapid/astrorapid/training_set_files/Figures/classify/ZTF__noAGN_batch500_unnormalised_epochs50_agTrue_ci(0,)_fpsaved_lc_MSIP_ZTF_20190512_firsttry.hdf5_zcut0.5_bcutTrue_varcutTrue/keras_model.hdf5' #'keras_model_with_redshift.hdf5'
        else:
            self.contextual_info = ()
            filename = 'keras_model_no_redshift.hdf5'

        if model_filepath != '' and os.path.exists(model_filepath):
            self.model_filepath = model_filepath
            print("Invalid keras model. Using default model...")
        else:
            self.model_filepath = os.path.join(SCRIPT_DIR, filename)
            if not os.path.exists(self.model_filepath):
                self.model_filepath = resource_filename(__name__, 'keras_model_with_redshift.hdf5')

        print(self.model_filepath)
        self.graph = graph
        if graph is not None and model is not None:
            self.model = model
        else:
            self.model = load_model(self.model_filepath)

    def process_light_curves(self, light_curves):
        processed_lightcurves = read_multiple_light_curves(light_curves, known_redshift=self.known_redshift,
                                                           training_set_parameters=None)
        prepareinputarrays = PrepareInputArrays(self.passbands, self.contextual_info, self.bcut, self.zcut)
        X, orig_lc, timesX, objids_list, trigger_mjds = prepareinputarrays.prepare_input_arrays(processed_lightcurves)

        # # REMOVE CORRECTION FACTOR IF NOT USED
        # correction_factor = np.load('astrorapid/correction_factor.npy')
        # for i, pb in enumerate(self.passbands):
        #     X[:, :, i] = X[:, :, i] / correction_factor[i]

        return X, orig_lc, timesX, objids_list, trigger_mjds

    def _do_error_checks(self, light_curves):
        assert isinstance(light_curves, (list, np.ndarray))
        for light_curve in light_curves:
            assert len(light_curve) == 10
            mjd, flux, fluxerr, passband, photflag, ra, dec, objid, redshift, mwebv = light_curve
            _lcshape = len(mjd)
            assert _lcshape == len(flux)
            assert _lcshape == len(fluxerr)
            assert _lcshape == len(passband)
            assert _lcshape == len(photflag)
            assert all(isinstance(n, numbers.Number) and np.isfinite(n) for n in mjd)
            assert all(isinstance(n, numbers.Number) and np.isfinite(n) for n in flux)
            assert all(isinstance(n, numbers.Number) and np.isfinite(n) for n in fluxerr)
            assert all(isinstance(n, numbers.Number) and np.isfinite(n) for n in photflag)
            assert all(isinstance(n, str) for n in passband)
            assert isinstance(ra, numbers.Number)
            assert isinstance(dec, numbers.Number)
            assert isinstance(mwebv, numbers.Number)
            assert np.isfinite(ra)
            assert np.isfinite(dec)
            assert np.isfinite(mwebv)
            if self.known_redshift:
                assert isinstance(redshift, numbers.Number)
                assert np.isfinite(redshift)

    def get_predictions(self, light_curves, return_predictions_at_obstime=False, return_objids=False):
        """ Return the classification accuracies as a function of time for each class

        Parameters
        ----------
        light_curves : list
            Is a list of tuples. Each tuple contains the light curve information of a transient object in the form
            (mjd, flux, fluxerr, passband, photflag, ra, dec, objid, redshift, mwebv).
            Here, mjd, flux, fluxerr, passband, and photflag are arrays.
            And ra, dec, objid, redshift, and mwebv are floats.
                mjd: list of Modified Julian Dates of light curve

                flux: list of fluxes at each mjd

                fluxerr: list of flux errors

                passband: list of strings indicating the passband. 'r' or 'g' for r-band or g-band observations.

                photflag: list of flags identifying whether the observation is a detection (4096), non-detection (0),
                or the first detection (6144).

                ra: Right Ascension (float value).

                dec: Declination (float value).

                objid: Object Identifier (String).

                redshift: Cosmological redshift of object (float). Set to NoneType if redshift is unknown.

                mwebv: Milky way extinction.

        return_predictions_at_obstime: bool
            Return the predictions at the observation times instead of at the 50 interpolated timesteps.
        return_objids : bool, optional
            If True, also return the object IDs (objids) in the same order as the returned predictions.

        Returns
        -------
        y_predict: array
            Classification probability vector at each time step for each object.
            Array of shape (s, n, m) is returned.
            Where s is the number of obejcts that are classified,
            n is the number of times steps, and m is the number of classes.
        time_steps: array
            MJD time steps corresponding to the timesteps of the y_predict array.
        objids : array, optional
            The object ids (objids) that were input into light_curves are returned in the same order as y_predict.
            Only provided if return_objids is True.
        """

        self._do_error_checks(light_curves)

        self.X, self.orig_lc, self.timesX, self.objids, self.trigger_mjds = self.process_light_curves(light_curves)
        nobjects = len(self.objids)

        if nobjects == 0:
            print("No objects to classify. These may have been removed from the chosen selection cuts")
            if return_objids:
                return None, None, self.objids
            else:
                return None, None

        if self.graph is not None:
            with self.graph.as_default():
                self.y_predict = self.model.predict(self.X)
        else:
            self.y_predict = self.model.predict(self.X)

        argmax = self.timesX.argmax(axis=1) + 1

        if return_predictions_at_obstime:
            (s, n, m) = self.y_predict.shape  # (s, n, m) = (num light curves, num timesteps, num classes)
            y_predict = []
            time_steps = []
            for idx in range(s):
                obs_time = []
                for pb in self.passbands:
                    if pb in self.orig_lc[idx]:
                        obs_time.append(self.orig_lc[idx][pb]['time'].values)
                obs_time = np.array(obs_time)
                obs_time = np.sort(obs_time[~np.isnan(obs_time)])
                y_predict_at_obstime = []
                for classnum, classname in enumerate(CLASS_NAMES):
                    y_predict_at_obstime.append(np.interp(obs_time, self.timesX[idx][:argmax[idx]], self.y_predict[idx][:, classnum][:argmax[idx]]))
                y_predict.append(np.array(y_predict_at_obstime).T)
                time_steps.append(obs_time + self.trigger_mjds[idx])
        else:
            y_predict = [self.y_predict[i][:argmax[i]] for i in range(nobjects)]
            time_steps = [self.timesX[i][:argmax[i]] + self.trigger_mjds[i] for i in range(nobjects)]

        if return_objids:
            return y_predict, time_steps, self.objids

        return y_predict, time_steps

    def plot_light_curves_and_classifications(self, indexes_to_plot=None, step=True, use_interp_flux=False, figdir='.',
                                              plot_matrix_input=False, light_curves=None):
        """
        Plot light curve (top panel) and classifications (bottom panel) vs time.

        Parameters
        ----------
        indexes_to_plot : tuple
            The indexes listed in the tuple will be plotted according to the order of the input light curves.
            E.g. (0, 1, 3, 5) will plot the zeroth, first, third and fifth light curves and classifications.
            If None or True, then all light curves will be plotted
        step : bool
            Plot step function along data points instead of interpolating classifications between data.
        use_interp_flux : bool
            Use all 50 timesteps when plotting classification probabilities rather than just at the timesteps with data.
        figdir : str
            Directory to save figure.
        plot_matrix_input : bool
            Plots the interpolated light curve passed into the neural network on top of the observations.
        light_curves : list
            This argument is only required if the get_predictions() method has not been run.
            Is a list of tuples. Each tuple contains the light curve information of a transient object in the form
            (mjd, flux, fluxerr, passband, photflag, ra, dec, objid, redshift, mwebv).
            Here, mjd, flux, fluxerr, passband, and photflag are arrays.
            ra, dec, objid, redshift, and mwebv are floats

        """

        font = {'family': 'normal',
                'size': 33}
        matplotlib.rc('font', **font)

        if indexes_to_plot is None or indexes_to_plot is True:
            indexes_to_plot = np.arange(len(self.y_predict))

        if not hasattr(self, 'y_predict'):
            self.get_predictions(light_curves)

        if plot_matrix_input:
            alpha_observations = 0.2
        else:
            alpha_observations = 1.
        for idx in indexes_to_plot:
            argmax = self.timesX[idx].argmax() + 1
            fig, (ax1, ax2) = plt.subplots(nrows=2, ncols=1, figsize=(13, 15),
                                           num="classification_vs_time_{}".format(idx), sharex=True)
            # ax1.axvline(x=0, color='k', linestyle='-', linewidth=1)
            # ax2.axvline(x=0, color='k', linestyle='-', linewidth=1)

            for pbidx, pb in enumerate(self.passbands):
                if pb in self.orig_lc[idx].keys():
                    ax1.errorbar(self.orig_lc[idx][pb]['time'], self.orig_lc[idx][pb]['flux'],
                                 yerr=self.orig_lc[idx][pb]['fluxErr'], fmt=PB_MARKER[pb], label=pb,
                                 c=PB_COLOR[pb], lw=3, markersize=10, alpha=alpha_observations)
                if plot_matrix_input:
                    ax1.plot(self.timesX[idx][:argmax], self.X[idx][:, pbidx][:argmax], c=PB_COLOR[pb], lw=3)

            new_t = np.array([self.orig_lc[idx][pb]['time'].values for pb in self.passbands if pb in self.orig_lc[idx]]).flatten()
            new_t = np.sort(new_t[~np.isnan(new_t)])
            if not use_interp_flux and not plot_matrix_input:
                new_y_predict = []
                for classnum, classname in enumerate(CLASS_NAMES):
                    new_y_predict.append(np.interp(new_t, self.timesX[idx][:argmax], self.y_predict[idx][:, classnum][:argmax]))

            for classnum, classname in enumerate(CLASS_NAMES):
                if use_interp_flux or plot_matrix_input:
                    ax2.plot(self.timesX[idx][:argmax], self.y_predict[idx][:, classnum][:argmax], '-', label=classname,
                             color=CLASS_COLOR[classname], linewidth=3)
                else:
                    if step:
                        ax2.step(new_t, new_y_predict[classnum], '-', label=classname, color=CLASS_COLOR[classname], linewidth=3, where='post')
                    else:
                        ax2.plot(new_t, new_y_predict[classnum], '-', label=classname, color=CLASS_COLOR[classname], linewidth=3,)

            ax1.legend(frameon=True, fontsize=33)#, loc='lower right')
            ax2.legend(frameon=True, fontsize=21.5)#, loc='center right')  # , ncol=2)
            ax2.set_xlabel("Days since trigger (rest frame)")  # , fontsize=18)
            ax1.set_ylabel("Relative Flux")  # , fontsize=15)
            ax2.set_ylabel("Class Probability")  # , fontsize=18)
            # ax1.set_ylim(-0.1, 1.1)
            ax2.set_ylim(0, 1)
            ax1.set_xlim(left=min(new_t))  # ax1.set_xlim(-70, 80)
            # ax1.grid(True)
            # ax2.grid(True)
            plt.setp(ax1.get_xticklabels(), visible=False)
            ax2.yaxis.set_major_locator(MaxNLocator(nbins=6, prune='upper'))  # added
            plt.tight_layout()
            fig.subplots_adjust(hspace=0)
            savename = 'classification_vs_time_{}{}{}.pdf'.format(self.objids[idx], '_step' if step else '', '_no_interp' if not use_interp_flux else '')
            plt.savefig(os.path.join(figdir, savename))
            # plt.savefig("{}.png".format(savename))
            plt.close()

        return self.orig_lc, self.timesX, self.y_predict

    def plot_classification_animation(self, indexes_to_plot=None, figdir='.', light_curves=None):
        """ Plot light curve (top panel) and classifications (bottom panel) vs time as an mp4 animation.

        Parameters
        ----------
        indexes_to_plot : tuple
            The indexes listed in the tuple will be plotted according to the order of the input light curves.
            E.g. (0, 1, 3, 5) will plot the zeroth, first, third and fifth light curves and classifications.
            If None or True, then all light curves will be plotted
        figdir : str
            Directory to save figure.
        light_curves : list
            This argument is only required if the get_predictions() method has not been run.
            Is a list of tuples. Each tuple contains the light curve information of a transient object in the form
            (mjd, flux, fluxerr, passband, photflag, ra, dec, objid, redshift, mwebv).
            Here, mjd, flux, fluxerr, passband, and photflag are arrays.
            ra, dec, objid, redshift, and mwebv are floats

        """

        font = {'family': 'normal',
                'size': 33}
        matplotlib.rc('font', **font)

        if indexes_to_plot is None or indexes_to_plot is True:
            indexes_to_plot = np.arange(len(self.y_predict))

        if not hasattr(self, 'y_predict'):
            self.get_predictions(light_curves)

        for idx in indexes_to_plot:
            new_t = np.array([self.orig_lc[idx][pb]['time'].values for pb in self.passbands if pb in self.orig_lc[idx]]).flatten()
            all_flux = list(self.orig_lc[idx]['g']['flux']) + list(self.orig_lc[idx]['r']['flux'])

            argmax = self.timesX[idx].argmax() + 1
            fig, (ax1, ax2) = plt.subplots(nrows=2, ncols=1, figsize=(13, 15),
                                           num="animation_classification_vs_time_{}".format(idx), sharex=True)

            ax1.legend(frameon=False, fontsize=33)
            ax2.legend(frameon=False, fontsize=23.5)  # , ncol=2)
            ax2.set_xlabel("Days since trigger (rest frame)")  # , fontsize=18)
            ax1.set_ylabel("Relative Flux")  # , fontsize=15)
            ax2.set_ylabel("Class Probability")  # , fontsize=18)
            ax1.set_ylim(min(all_flux), max(all_flux))
            ax2.set_ylim(0, 1)
            ax1.set_xlim(min(new_t), max(new_t))  # ax1.set_xlim(-70, 80)
            plt.setp(ax1.get_xticklabels(), visible=False)
            ax2.yaxis.set_major_locator(MaxNLocator(nbins=6, prune='upper'))  # added
            plt.tight_layout()
            fig.subplots_adjust(hspace=0)
            ax1.axvline(x=0, color='k', linestyle='-', linewidth=1)
            ax2.axvline(x=0, color='k', linestyle='-', linewidth=1)

            Writer = animation.writers['ffmpeg']
            writer = Writer(fps=5, bitrate=1800)

            def animate(i):
                for pbidx, pb in enumerate(self.passbands):
                    ax1.plot(self.timesX[idx][:argmax][:int(i + 1)], self.X[idx][:, pbidx][:argmax][:int(i + 1)],
                             label=pb, c=PB_COLOR[pb], lw=3)  # , markersize=10, marker=MARKPB[pb])

                for classnum, classname in enumerate(CLASS_NAMES):
                    ax2.plot(self.timesX[idx][:argmax][:int(i + 1)],
                             self.y_predict[idx][:, classnum][:argmax][:int(i + 1)],
                             '-', label=classname, color=CLASS_COLOR[classname], linewidth=3)

                # Don't repeat legend items
                ax1.legend(frameon=False, fontsize=33)
                ax2.legend(frameon=False, fontsize=23.5)
                handles1, labels1 = ax1.get_legend_handles_labels()
                handles2, labels2 = ax2.get_legend_handles_labels()
                by_label1 = OrderedDict(zip(labels1, handles1))
                by_label2 = OrderedDict(zip(labels2, handles2))
                ax1.legend(by_label1.values(), by_label1.keys(), frameon=False, fontsize=33, loc='lower right')
                ax2.legend(by_label2.values(), by_label2.keys(), frameon=False, fontsize=21.5, loc='center right')

            ani = animation.FuncAnimation(fig, animate, frames=50, repeat=True)
            savename = os.path.join(figdir,'classification_vs_time_{}.mp4'.format(self.objids[idx]))
            ani.save(savename, writer=writer)

    def plot_classification_animation_step(self, indexes_to_plot=None, figdir='.', light_curves=None):
        """
        Plot light curve (top panel) and classifications (bottom panel) vs time as an mp4 animation
        as step function.

        Parameters
        ----------
        indexes_to_plot : tuple
            The indexes listed in the tuple will be plotted according to the order of the input light curves.
            E.g. (0, 1, 3, 5) will plot the zeroth, first, third and fifth light curves and classifications.
            If None or True, then all light curves will be plotted
        figdir : str
            Directory to save figure.
        light_curves : list
            This argument is only required if the get_predictions() method has not been run.
            Is a list of tuples. Each tuple contains the light curve information of a transient object in the form
            (mjd, flux, fluxerr, passband, photflag, ra, dec, objid, redshift, mwebv).
            Here, mjd, flux, fluxerr, passband, and photflag are arrays.
            ra, dec, objid, redshift, and mwebv are floats

        """

        font = {'family': 'normal',
                'size': 33}
        matplotlib.rc('font', **font)

        if indexes_to_plot is None or indexes_to_plot is True:
            indexes_to_plot = np.arange(len(self.y_predict))

        if not hasattr(self, 'y_predict'):
            self.get_predictions()

        for idx in indexes_to_plot:
            new_t = np.array([self.orig_lc[idx][pb]['time'].values for pb in self.passbands]).flatten()
            new_t = np.sort(new_t[~np.isnan(new_t)])
            new_y_predict = []
            all_flux = list(self.orig_lc[idx]['g']['flux']) + list(self.orig_lc[idx]['r']['flux'])

            argmax = self.timesX[idx].argmax() + 1
            fig, (ax1, ax2) = plt.subplots(nrows=2, ncols=1, figsize=(13, 15),
                                           num="animation_classification_vs_time_{}".format(idx), sharex=True)

            ax1.legend(frameon=False, fontsize=33)
            ax2.legend(frameon=False, fontsize=23.5)  # , ncol=2)
            ax2.set_xlabel("Days since trigger (rest frame)")  # , fontsize=18)
            ax1.set_ylabel("Relative Flux")  # , fontsize=15)
            ax2.set_ylabel("Class Probability")  # , fontsize=18)
            ax1.set_ylim(min(all_flux), max(all_flux))
            ax2.set_ylim(0, 1)
            ax1.set_xlim(min(new_t), max(new_t))
            plt.setp(ax1.get_xticklabels(), visible=False)
            ax2.yaxis.set_major_locator(MaxNLocator(nbins=6, prune='upper'))  # added
            plt.tight_layout()
            fig.subplots_adjust(hspace=0)
            ax1.axvline(x=0, color='k', linestyle='-', linewidth=1)
            ax2.axvline(x=0, color='k', linestyle='-', linewidth=1)

            Writer = animation.writers['ffmpeg']
            writer = Writer(fps=2, bitrate=1800)

            for classnum, classname in enumerate(CLASS_NAMES):
                new_y_predict.append(np.interp(new_t, self.timesX[idx][:argmax], self.y_predict[idx][:, classnum][:argmax]))

            def animate(i):
                for pbidx, pb in enumerate(self.passbands):
                    # ax1.plot(self.timesX[idx][:argmax][:int(i + 1)], self.X[idx][:, pbidx][:argmax][:int(i + 1)],
                    #          label=pb, c=PB_COLOR[pb], lw=3)  # , markersize=10, marker=MARKPB[pb])

                    # dea = [self.orig_lc[idx][pb]['time'] < self.timesX[idx][:argmax][int(i)]]
                    if i + 1 >= len(new_t):
                        break

                    dea = [self.orig_lc[idx][pb]['time'] < new_t[int(i+1)]]

                    ax1.errorbar(np.array(self.orig_lc[idx][pb]['time'])[dea], np.array(self.orig_lc[idx][pb]['flux'])[dea],
                                 yerr=np.array(self.orig_lc[idx][pb]['fluxErr'])[dea], fmt=PB_MARKER[pb], label=pb,
                                 c=PB_COLOR[pb], lw=3, markersize=10)

                for classnum, classname in enumerate(CLASS_NAMES):
                    # ax2.plot(self.timesX[idx][:argmax][:int(i + 1)],
                    #          self.y_predict[idx][:, classnum][:argmax][:int(i + 1)],
                    #          '-', label=classname, color=CLASS_COLOR[classname], linewidth=3)
                    ax2.step(new_t[:int(i + 1)],
                             new_y_predict[classnum][:int(i + 1)],
                             '-', label=classname, color=CLASS_COLOR[classname], linewidth=3, where='post')

                # Don't repeat legend items
                ax1.legend(frameon=False, fontsize=33)
                ax2.legend(frameon=False, fontsize=23.5)
                handles1, labels1 = ax1.get_legend_handles_labels()
                handles2, labels2 = ax2.get_legend_handles_labels()
                by_label1 = OrderedDict(zip(labels1, handles1))
                by_label2 = OrderedDict(zip(labels2, handles2))
                ax1.legend(by_label1.values(), by_label1.keys(), frameon=False, fontsize=33, loc='lower right')
                ax2.legend(by_label2.values(), by_label2.keys(), frameon=False, fontsize=21.5, loc='center right')

            ani = animation.FuncAnimation(fig, animate, frames=len(new_t), repeat=True)
            savename = os.path.join(figdir, 'classification_vs_time_{}_step.mp4'.format(self.objids[idx]))
            ani.save(savename, writer=writer)




import sys
import os
import configparser
import pymysql
import getpass

# we should probably get these from a file instead
_MYSQL_CONFIG = {'host':'dlgreenmysqlv.stsci.edu',
                'port':43306,
                'user':'gnarayan',
                'database':'yse',
                'password':None}


def get_mysql_config():
    mysql_setting_file = os.path.expanduser(os.path.join('~','.my.cnf'))
    if os.path.exists(mysql_setting_file):
        reader = configparser.RawConfigParser()
        reader.read(mysql_setting_file)
        if 'client' in reader.sections():
            settings = dict(reader['client'].items())
            for key, old_val in _MYSQL_CONFIG.items():
                if old_val is None:
                    new_val = settings.get(key)
                    if new_val is not None:
                        _MYSQL_CONFIG[key] = new_val
    else:
        print("Create a ~/.my.cnf if you don't want to keep being bugged about the password.")
    return _MYSQL_CONFIG                


def make_mysql_schema_from_astropy_bintable_cols(astropy_columns):
    """
    Builds a schema for the data in the HEAD.fits file, converting from numpy
    types to mysql datatypes
    """
    fields = astropy_columns.names

    # the header files have some columns that are model dependent, so they
    # aren't present for all models. This eliminates those columns from the
    # mysql, so we are guaranteed to always have common columns for all files.
    # The price is that we will have to get these model specific columns from
    # the HEAD.fits, but it's less painful to do that for just a few objects
    # than to store several NULLs for all objects in mysql.
    use_fields = []
    for field in fields:
        if field.startswith('SIMSED')\
            or field.startswith('LCLIB')\
            or field.startswith('SIM_TEMPLATE')\
            or field.startswith('SIM_SALT2')\
            or field.startswith('SIM_GALFRAC'):
            continue
        else:
            use_fields.append(field)

    # upper case fieldnames are THE WORST.
    mysql_fields = ['objid',] + [field.lower().replace(')','').replace('(','_') for field in use_fields]

    # this is the FITS format codes
    # FITS format code         Description                     8-bit bytes
    # L                        logical (Boolean)               1
    # X                        bit                             *
    # B                        Unsigned byte                   1
    # I                        16-bit integer                  2
    # J                        32-bit integer                  4
    # K                        64-bit integer                  4
    # A                        character                       1
    # E                        single precision floating point 4
    # D                        double precision floating point 8
    # C                        single precision complex        8
    # M                        double precision complex        16
    # P                        array descriptor                8
    # Q                        array descriptor                16

    # we don't need to support all the FITS codes - just the subset that we're going to see in the data
    dtype_conversion = {'I':'INTEGER',
                        'J':'BIGINT',
                        'K':'BIGINT',
                        'A':'TINYTEXT',
                        'E':'FLOAT',
                        'D':'DOUBLE'}

    mysql_formats = ['VARCHAR(255)',] + [dtype_conversion.get(x[-1], 'TINYTEXT') for x in astropy_columns.formats]
    mysql_schema = ', '.join(['{} {}'.format(x, y) for x, y in zip(mysql_fields, mysql_formats)])
    return use_fields, mysql_fields, mysql_schema


def get_sql_password():
    """
    Prompts the user for a mysql password 
    Only accepts input from specified users
    """
    user = None
    try:
        user = getpass.getuser()
        password = getpass.getpass(prompt='Enter MySQL password: ', stream=sys.stderr)
        if user not in ['gnarayan', 'dmuthukrishna']:
            message = 'Unauthorized user {}'.format(user)
            raise RuntimeError(message)
        _MYSQL_CONFIG['password'] = password
    except Exception as e:
        message = '{}\nCould not get password from user {}'.format(e, user)
        raise RuntimeError(message)


def check_sql_db_for_table(table_name):
    """
    Check if a MySQL Table exists for this data_release
    """
    query = 'show tables'
    result = exec_sql_query(query)
    existing_tables = [table[0] for table in result]
    if table_name in existing_tables:
        return True 
    else:
        return False


def drop_sql_table_from_db(table_name):
    """
    Drops a MySQL Table from the table (assumes it exists)
    """
    query = 'drop table {}'.format(table_name)
    result = exec_sql_query(query)
    return result


def get_index_table_name_for_release(data_release):
    """
    Some datareleases are defined purely as ints which make invalid MySQL table names
    Fix that here by always prefixing release_ to the data_release name to make the MySQL table_name
    """
    table_name = 'release_{}'.format(data_release)
    return table_name


def create_sql_index_table_for_release(data_release, schema, redo=False, table_name=None):
    """
    Creates a MySQL Table to hold useful data from HEAD.fits files from the
    PLASTICC sim. Checks if the table exists already. The schema for this
    release must be supplied. This is not hardcoded since, other than SNID, we
    do not expect the schema to be the same for all data releases.
    Drops table if redo.
    Returns a table_name
    """
    if not table_name:
        table_name = get_index_table_name_for_release(data_release)
    result = check_sql_db_for_table(table_name)
    if result:
        print("Table {} exists.".format(table_name))
        if redo:
            print("Clobbering table {}.".format(table_name))
            drop_sql_table_from_db(table_name)
        else:
            return table_name

    query = 'CREATE TABLE {} ({}, PRIMARY KEY (objid))'.format(table_name, schema)
    result =  exec_sql_query(query)
    print("Created Table {}.".format(table_name))
    return table_name


def write_rows_to_index_table(index_entries, table_name):
    """
    Write rows to an index table
    Returns number of rows written
    """
    primitive = ['%s',]
    nrows = len(index_entries)
    if nrows == 0:
        return nrows

    ncols = len(index_entries[0])
    format_string = ', '.join(primitive*ncols)

    con = get_mysql_connection()
    query = f'INSERT INTO {table_name} VALUES ({format_string})'
    cursor = con.cursor()
    number_of_rows = cursor.executemany(query, index_entries)
    con.commit()
    con.close()
    return number_of_rows


def get_mysql_connection(attempts=0):
    """
    Get a  MySQL connection object. The config variable _MYSQL_CONFIG defines
    the context/parameters of the connection. If config does not include a
    MySQL password, the user is prompted for it.
    Returns a MySQL connection object.
    """
    _MYSQL_CONFIG = get_mysql_config() 
    password = _MYSQL_CONFIG.get('password')
    if password is None:
        get_sql_password()
    try:
        con = pymysql.connect(**_MYSQL_CONFIG)
    except Exception as e:
        attempts += 1
        if attempts < 3:
            _MYSQL_CONFIG.pop('password')
            con = get_mysql_connection(attempts=attempts)
        else:
            message = 'Login attempts exceeded!'
            raise RuntimeError(message)
    return con


def exec_big_sql_query(query, big=False):
    """
    Executes a supplied MySQL query. The context of the query is defined by
    _MYSQL_CONFIG and the connection object returned by get_mysql_connection() 
    Returns the result of the query (if any) or yields it as a generator if big=True
    """
    print(query)
    result = None
    con = get_mysql_connection()
    try:
        cursor = con.cursor()
        success = cursor.execute(query)
        print('Query results:',success)
        loop_ok = True 
        while loop_ok:
            result = cursor.fetchone()
            if result:
                yield result 
            else:
                loop_ok = False
    except Exception as e:
        message = '{}\nFailed to execute query\n{}'.format(e, query)
        raise RuntimeError(message)
    finally:
        con.close()
    return result

def exec_sql_query(query, big=False):
    """
    Executes a supplied MySQL query. The context of the query is defined by
    _MYSQL_CONFIG and the connection object returned by get_mysql_connection() 
    Returns the result of the query (if any) or yields it as a generator if big=True
    """
    print(query)
    result = None
    con = get_mysql_connection()
    try:
        cursor = con.cursor()
        success = cursor.execute(query)
        print('Query results:',success)
        result = cursor.fetchall()
    except Exception as e:
        message = '{}\nFailed to execute query\n{}'.format(e, query)
        raise RuntimeError(message)
    finally:
        con.close()
    return result



import numpy as np
from collections import OrderedDict
import json
from six.moves.urllib.request import urlopen
from astroquery.irsa_dust import IrsaDust
import astropy.coordinates as coord
import astropy.units as u


from astrorapid.classify import Classify


def read_json(url):
    response = urlopen(url)
    return json.loads(response.read(), object_pairs_hook=OrderedDict)


def delete_indexes(deleteindexes, *args):
    newarrs = []
    for arr in args:
        newarr = np.delete(arr, deleteindexes)
        newarrs.append(newarr)

    return newarrs


def convert_lists_to_arrays(*args):
    output = []
    for arg in args:
        out_array = np.asarray(arg)
        output.append(out_array)

    return output


def read_lasair_json(object_name='ZTF18acsovsw'):
    """
    Read light curve from lasair website API based on object name.

    Parameters
    ----------
    object_name : str
        The LASAIR object name. E.g. object_name='ZTF18acsovsw'

    """
    print(object_name)
    if isinstance(object_name, tuple):
        object_name, z_in = object_name
    else:
        z_in = None

    url = 'https://lasair.roe.ac.uk/object/{}/json/'.format(object_name)

    data = read_json(url)

    objid = data['objectId']
    ra = data['objectData']['ramean']
    dec = data['objectData']['decmean']
    # lasair_classification = data['objectData']['classification']
    tns_info = data['objectData']['annotation']
    photoZ = None
    for cross_match in data['crossmatches']:
        # print(cross_match)
        photoZ = cross_match['photoZ']
        separation_arcsec = cross_match['separationArcsec']
        catalogue_object_type = cross_match['catalogue_object_type']
    if z_in is not None:
        redshift = z_in
    else:
        if photoZ is None:  # TODO: Get correct redshift
            try:
                if "z=" in tns_info:
                    photoZ = tns_info.split('z=')[1]
                    redshift = float(photoZ.replace(')', '').split()[0])
                    # print("PHOTOZZZZZZZZZZZZ", redshift, tns_info)
                elif "Z=" in tns_info:
                    photoZ = tns_info.split('Z=')[1]
                    redshift = float(photoZ.split()[0])
                    # print("PHOTOZZZZZZZZZZZZ", redshift, tns_info)
                else:
                    # return
                    print("TRYING ARBITRARY GUESS REDSHIFT = 0.1")
                    redshift = None
            except Exception as e:
                # return
                redshift = None
                print(e)
        else:
            redshift = photoZ

    print("Redshift is {}".format(redshift))
    if redshift is not None:
        objid += "_z={}".format(round(redshift, 2))

    # Get extinction  TODO: Maybe add this to RAPID code
    coo = coord.SkyCoord(ra * u.deg, dec * u.deg, frame='icrs')
    dust = IrsaDust.get_query_table(coo, section='ebv')
    mwebv = dust['ext SandF mean'][0]
    print("MWEBV")
    print(mwebv)

    mjd = []
    passband = []
    mag = []
    magerr = []
    photflag = []
    zeropoint = []
    dc_mag = []
    dc_magerr = []
    magnr, sigmagnr, isdiffpos = [], [], []
    for cand in data['candidates']:
        mjd.append(cand['mjd'])
        passband.append(cand['fid'])
        mag.append(cand['magpsf'])
        if 'sigmapsf' in cand:
            magerr.append(cand['sigmapsf'])
            photflag.append(4096)

            # if cand['magzpsci'] == 0:
            #     print("NO ZEROPOINT")
            #      zeropoint.append(26.2)  # TODO: Tell LASAIR their zeropoints are wrong
            # else:
            #     zeropoint.append(cand['magzpsci'])  #26.2) #
            zeropoint.append(cand['magzpsci'])
            dc_mag.append(cand['dc_mag'])
            dc_magerr.append(cand['dc_sigmag'])
            magnr.append(cand['magnr'])
            sigmagnr.append(cand['sigmagnr'])
            isdiffpos.append(cand['isdiffpos'])
        else:
            magerr.append(np.nan)#0.01 * cand['magpsf'])  #magerr.append(None)  #magerr.append(0.1 * cand['magpsf'])  #
            photflag.append(0)
            zeropoint.append(np.nan)#26.2)
            dc_mag.append(np.nan)
            dc_magerr.append(np.nan)
            magnr.append(np.nan)
            sigmagnr.append(np.nan)
            isdiffpos.append(None)

    mjd, passband, mag, magerr, photflag, zeropoint, dc_mag, dc_magerr, magnr, sigmagnr, isdiffpos = convert_lists_to_arrays(mjd, passband, mag, magerr, photflag, zeropoint, dc_mag, dc_magerr, magnr, sigmagnr, isdiffpos)

    # deleteindexes = np.where(magerr == None)  # [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29]  #
    # mjd, passband, mag, magerr, photflag, zeropoint, dc_mag, dc_magerr, magnr, sigmagnr, isdiffpos = delete_indexes(deleteindexes, mjd, passband, mag, magerr, photflag, zeropoint, dc_mag, dc_magerr, magnr, sigmagnr, isdiffpos)
    deleteindexes = np.where((photflag==0) & (mjd > min(mjd[photflag>0])))  # delete where nondetections after first detection
    mjd, passband, mag, magerr, photflag, zeropoint, dc_mag, dc_magerr, magnr, sigmagnr, isdiffpos = delete_indexes(deleteindexes, mjd, passband, mag, magerr, photflag, zeropoint, dc_mag, dc_magerr, magnr, sigmagnr, isdiffpos)
    deleteindexes = np.where((mag < (np.median(mag[photflag==0]) - 0.5*np.std(mag[photflag==0]))) & (photflag==0)) # Remove non detection outliers
    mjd, passband, mag, magerr, photflag, zeropoint, dc_mag, dc_magerr, magnr, sigmagnr, isdiffpos = delete_indexes(deleteindexes, mjd, passband, mag, magerr, photflag, zeropoint, dc_mag, dc_magerr, magnr, sigmagnr, isdiffpos)

    return mjd, passband, mag, magerr, photflag, zeropoint, ra, dec, objid, redshift, mwebv, dc_mag, dc_magerr, magnr, sigmagnr, isdiffpos


def classify_lasair_light_curves(object_names=('ZTF18acsovsw',), plot=True, figdir='.'):
    light_curve_list = []
    peakfluxes_g, peakfluxes_r = [], []
    mjds, passbands, mags, magerrs, zeropoints, photflags = [], [], [], [], [], []
    dc_mags, dc_magerrs, magnrs, sigmagnrs, isdiffposs = [], [], [], [], []
    obj_names = []
    ras, decs, objids, redshifts, mwebvs = [], [], [], [], []
    peakmags_g, peakmags_r = [], []
    for object_name in object_names:
        try:
            mjd, passband, mag, magerr, photflag, zeropoint, ra, dec, objid, redshift, mwebv, dc_mag, dc_magerr, magnr, sigmagnr, isdiffpos = read_lasair_json(object_name)
            sortidx = np.argsort(mjd)
            mjds.append(mjd[sortidx])
            passbands.append(passband[sortidx])
            mags.append(mag[sortidx])
            magerrs.append(magerr[sortidx])
            zeropoints.append(zeropoint[sortidx])
            photflags.append(photflag[sortidx])
            dc_mags.append(dc_mag[sortidx])
            dc_magerrs.append(dc_magerr[sortidx])
            magnrs.append(magnr[sortidx])
            sigmagnrs.append(sigmagnr[sortidx])
            isdiffposs.append(isdiffpos[sortidx])

            obj_names.append(object_name)
            ras.append(ra)
            decs.append(dec)
            objids.append(objid)
            redshifts.append(redshift)
            mwebvs.append(mwebv)
            peakmags_g.append(min(mag[passband==1]))
            peakmags_r.append(min(mag[passband==2]))

        except Exception as e:
            print(e)
            continue

        flux = 10. ** (-0.4 * (mag - zeropoint))
        fluxerr = np.abs(flux * magerr * (np.log(10.) / 2.5))

        passband = np.where((passband == 1) | (passband == '1'), 'g', passband)
        passband = np.where((passband == 2) | (passband == '2'), 'r', passband)

        # Set photflag detections when S/N > 5
        photflag2 = np.zeros(len(flux))
        photflag2[flux / fluxerr > 5] = 4096
        photflag2[np.where(mjd == min(mjd[photflag2 == 4096]))] = 6144

        mjd_first_detection = min(mjd[photflag == 4096])
        photflag[np.where(mjd == mjd_first_detection)] = 6144

        deleteindexes = np.where(((passband == 3) | (passband == '3')) | (mjd > mjd_first_detection) & (photflag == 0))
        if deleteindexes[0].size > 0:
            print("Deleting indexes {} at mjd {} and passband {}".format(deleteindexes, mjd[deleteindexes], passband[deleteindexes]))
        mjd, passband, flux, fluxerr, zeropoint, photflag = delete_indexes(deleteindexes, mjd, passband, flux, fluxerr, zeropoint, photflag)

        light_curve_list += [(mjd, flux, fluxerr, passband, photflag, ra, dec, objid, redshift, mwebv)]

        try:
            dummy = max(flux[passband == 'g'])
            dummy = max(flux[passband == 'r'])
        except Exception as e:
            print("Error", e)
            continue

        peakfluxes_g.append(max(flux[passband == 'g']))
        peakfluxes_r.append(max(flux[passband == 'r']))

    import sys
    import pickle
    with open('save_real_ZTF_unprocessed_data_snia_osc_12nov2019.npz', 'wb') as f:
        pickle.dump([mjds, passbands, mags, magerrs, photflags, zeropoints, dc_mags, dc_magerrs, magnrs, sigmagnrs, isdiffposs, ras, decs, objids, redshifts, mwebvs], f)
    # np.savez('save_real_ZTF_unprocessed_data_snia_osc_12nov2019.npz', mjds=mjds, passbands=passbands, mags=mags, magerrs=magerrs, photflags=photflags, zeropoints=zeropoints, ras=ras, decs=decs, objids=objids, redshifts=redshifts, mwebvs=mwebvs)# , peakflux_g=peakfluxes_g, peakflux_r=peakfluxes_r)
    print("finished")
    sys.exit(0)
    with open('save_real_ZTF_unprocessed_data_snia_osc_12nov2019.npz', 'rb') as f:
        a = pickle.load(f)

    classification = Classify(known_redshift=True, bcut=False, zcut=None)
    predictions, time_steps = classification.get_predictions(light_curve_list, return_predictions_at_obstime=False)
    print(predictions)

    if plot:
        try:
            classification.plot_light_curves_and_classifications(step=True, use_interp_flux=False, figdir=figdir, plot_matrix_input=True)
        except Exception as e:
            print(e)
        # classification.plot_light_curves_and_classifications(step=False, use_interp_flux=True)
        # classification.plot_light_curves_and_classifications(step=False, use_interp_flux=False)
        # classification.plot_classification_animation_step()
        # classification.plot_classification_animation()



    return classification.orig_lc, classification.timesX, classification.y_predict


if __name__ == '__main__':
    # classify_lasair_light_curves(object_name='ZTF19aabbnzo')
    # classify_lasair_light_curves(object_name='ZTF18abukavn')
    # classify_lasair_light_curves(object_name='ZTF18aajupnt') # 2018dyk
    # classify_lasair_light_curves(object_name='ZTF18aabtxvd') # 2018zr
    # classify_lasair_light_curves(object_name='ZTF18acpdvos') # 2018hyz

    # classify_lasair_light_curves(object_names=['ZTF19abraqpf', 'ZTF19abqstxq', ('ZTF19abpmetl', 0.068), 'ZTF19abgsssu', 'ZTF19abdooly'], figdir='astrorapid')
    # classify_lasair_light_curves(object_names=[('ZTF18aazblzy', 0.064), 'ZTF19aabyppp'], figdir='astrorapid')
    # classify_lasair_light_curves(object_names=[('ZTF19abzrhgq', 0.0151), ('ZTF19abuhlxk', 0.02), ('ZTF19abylxyt', 0.0533)], figdir='astrorapid') # (2019qiz TDE), (2019pdx SNIa), (2019qid SNIa)


    # classify_lasair_light_curves(object_names=('ZTF18acmxkzj', 'ZTF18adbntwo', 'ZTF18acmzpbf', 'ZTF18acrdmmw', 'ZTF18acsouxk', 'ZTF18aczrafz', 'ZTF18aceynvm', 'ZTF18acdzzyf', 'ZTF18acybdar', 'ZTF18ablhvfr', 'ZTF18acebefc', 'ZTF18acslpba', 'ZTF18aaxcxih', 'ZTF18aaxcntm', 'ZTF18abspqsn', 'ZTF18abvejqt','ZTF18adbntwo', 'ZTF18adasopt', 'ZTF18aajupnt', 'ZTF18aaxkqgy', 'ZTF18abckutn', 'ZTF18abrlljc', 'ZTF18acyybvg', 'ZTF19aadnmgf', 'ZTF18abxftqm',))

    # orig_lc, timesX, y_predict = classify_lasair_light_curves(object_names=[
    #                                                             'ZTF18absgnio', 'ZTF19abxpfxx','ZTF19abvhtsk','ZTF19abyheza', 'ZTF19abyukuy', 'ZTF18acaqbgu',
    #                                                             # ('ZTF19abzcaod', 0.052098),
    #                                                             # 'ZTF19abgjoth',
    #                                                             # 'ZTF19abhhjcc',
    #                                                             # 'ZTF19aazcxwk', 'ZTF19abauzma',
    #                                                             # 'ZTF18abxftqm',  # TDE
    #                                                             # 'ZTF19aadnmgf',  # SNIa
    #                                                             # 'ZTF18acmzpbf',  # SNIa
    #                                                             # 'ZTF19aakzwao',  # SNIa
    #                                                             ], figdir='astrorapid')


    # orig_lc, timesX, y_predict = classify_lasair_light_curves(object_names=[('ZTF19aarioci', 0.074), ('ZTF19aabbnzo', 0.08), ('ZTF19aabyppp', 0.037)], figdir='astrorapid') # ( TDE), ( SNIa), ( SNIa) ('ZTF19aatlmbo', 0.007755), ('ZTF19aabyppp', 0.037), ('ZTF19aabyppp', 0.08), ('ZTF19aadnxat', 0.03)
    # import pickle
    # with open('paper_plot_real_data_new2019.pickle', 'wb') as f:
    #     pickle.dump([orig_lc, timesX, y_predict], f)

    # tde_candidates = [('ZTF19aapreis', 0.0512),
    #                   'ZTF18actaqdw',
    #                   ('ZTF18aahqkbt', 0.051),
    #                   ('ZTF18abxftqm', 0.09),
    #                   ('ZTF19aabbnzo', 0.08),
    #                   'ZTF18acpdvos',
    #                   'ZTF18aabtxvd',
    #                   ('ZTF19aarioci', 0.12)]
    # classify_lasair_light_curves(object_names=tde_candidates, figdir='astrorapid')


    # # nuclear_transients =  ('ZTF19aalzmdv', ('ZTF19aapreis', 0.0512),'ZTF19aandznq', ('ZTF18abxftqm', 0.09), ('ZTF19aarioci', 0.12),'ZTF19aalveag', 'ZTF18aajgjow', 'ZTF19aaluprf', 'ZTF19aamrzte', 'ZTF18achxcsh', 'ZTF19aalzmio', 'ZTF19aamhhae', 'ZTF19aamkowf', 'ZTF18aaizdjz', 'ZTF19aamhhgu', 'ZTF19aamvkpf', 'ZTF19aamfxhg', ('ZTF19aabbnzo', 0.08), 'ZTF19aanekbm', 'ZTF19aaneseo', 'ZTF19aamkgiw', 'ZTF19aanevry', 'ZTF19aanfyey', 'ZTF19aamqlok', 'ZTF19aamquei', 'ZTF19aamszrg', 'ZTF19aanfkuf', 'ZTF19aamhbqx', 'ZTF19aanickg', 'ZTF19aanikcm', 'ZTF19aamqveb', 'ZTF19aaniiaz', 'ZTF19aalzmae', 'ZTF19aamvmer', 'ZTF19aaniibk', 'ZTF19aamkmxv', 'ZTF19aamuxso', 'ZTF19aamvavq', 'ZTF19aanbojt', 'ZTF19aanflwj', 'ZTF19aangfld', 'ZTF19aamtwwv', 'ZTF19aanlews', 'ZTF19aanuuzg', 'ZTF19aamtjzm', 'ZTF19aamkywr', 'ZTF19aangzrq', 'ZTF19aaomlmm', 'ZTF19aanuuen', 'ZTF19aanlvnl', 'ZTF19aanfrmq', 'ZTF19aanetjx', 'ZTF19aaoxijx', 'ZTF19aanetcr', 'ZTF19aaneugz', 'ZTF18aahtjsc', 'ZTF19aaohwqp', 'ZTF19aanhfnk', 'ZTF19aaogtqk', 'ZTF19aaozmrs', 'ZTF19aaoznau', 'ZTF19aaojnrx', 'ZTF19aamqjqc', 'ZTF19aaokcsn', 'ZTF19aalarng', 'ZTF18aailsok', 'ZTF19aaocnda', 'ZTF19aapafew', 'ZTF19aaondau', 'ZTF18aaivonj', 'ZTF19aanxosu', 'ZTF19aantojb', 'ZTF19aamctcx', 'ZTF19aapadsm', 'ZTF19aaoubqg', 'ZTF19aaohaau', 'ZTF19aaoibgs', 'ZTF19aaohasv', 'ZTF19aaohbqk', 'ZTF19aanevcu', 'ZTF19aapfpnz', 'ZTF19aampaby', 'ZTF19aapkjmd', 'ZTF18acnbexb', 'ZTF19aamkrcp', 'ZTF19aaoznax', 'ZTF19aaprgvu', 'ZTF19aanhcyf', 'ZTF19aaoxzej', 'ZTF19aaneuil', 'ZTF19aaoyinh', 'ZTF19aapdbau', 'ZTF19aanxnkd', 'ZTF19aaoigdw', 'ZTF19aaomnkb', 'ZTF19aaopqig', 'ZTF19aapamsd', 'ZTF18abfcmjw', 'ZTF19aanxrkt', 'ZTF19aadopob', 'ZTF19aaoakjo', 'ZTF19aanfzox', 'ZTF19aapsxfl', 'ZTF19aapawxp', 'ZTF19aaptarf', 'ZTF19aapswkx', 'ZTF19aapbfot', 'ZTF19aapavln', 'ZTF19aapbbso', 'ZTF19aaqrlmz', 'ZTF18abldizg', 'ZTF19aapvltt', 'ZTF19aapxlhm',  'ZTF19aarhrru', 'ZTF19aarhrxs', 'ZTF19aapctvv', 'ZTF18aasxzyb', 'ZTF18aawwmga', 'ZTF18aalxmmk', 'ZTF19aaoycfk', 'ZTF19aarifvy', 'ZTF19aaraxhh', 'ZTF19aaqdfwd', 'ZTF19aanjryl', 'ZTF19aarfksi', 'ZTF19aariuqe', 'ZTF19aarjbyj', 'ZTF19aariwpf', 'ZTF19aapusqe', 'ZTF19aarinmw', 'ZTF19aapwoid', 'ZTF18abddtca', 'ZTF19aaricev', 'ZTF19aaqtcmy', 'ZTF19aarkuwd', 'ZTF19aarflsx', 'ZTF19aaqgxpy', 'ZTF19aariudl', 'ZTF19aaqhelq', 'ZTF19aarnrfo', 'ZTF19aarnmqs', 'ZTF18aamuasc', 'ZTF19aaqwwux', 'ZTF19aaqwwvh', 'ZTF19aarhphl', 'ZTF19aaqwtbg', 'ZTF19aaoxlzb', 'ZTF19aaojljx', 'ZTF19aarinqi', 'ZTF19aarmzoi', 'ZTF19aarhyjx', 'ZTF19aarisyg', 'ZTF19aarisna', 'ZTF19aasbphu', 'ZTF19aarhlrp', 'ZTF19aarrqat', 'ZTF19aarrufq', 'ZTF18aceqtdz', 'ZTF18abjjgiz', 'ZTF18aawodxu', 'ZTF19aasensj', 'ZTF18aanxnrz', 'ZTF19aatfkmb', 'ZTF19aarnhuv', 'ZTF19aatgrxv', 'ZTF19aarrcmx', 'ZTF19aarqpeb', 'ZTF19aatjrua', 'ZTF19aarsact', 'ZTF18aaxokeg', 'ZTF19aariuvr', 'ZTF18aaxisdq', 'ZTF19aasejuy', 'ZTF19aatgrxl', 'ZTF19aatfkms', 'ZTF19aasbqaq', 'ZTF19aarfreu', 'ZTF19aascqge', 'ZTF19aascpnd', 'ZTF19aarisvo', 'ZTF18aaqkdwu', 'ZTF19aarbbfu', 'ZTF19aaqesdr', 'ZTF19aatqugy', 'ZTF19aaprehw', 'ZTF19aaafyjr', 'ZTF19aaqtxaf', 'ZTF19aarqihw', 'ZTF19aarfequ', 'ZTF19aatgaiq', 'ZTF19aarspxq', 'ZTF19aaskaxz', 'ZTF19aatgopg', 'ZTF19aathate', 'ZTF19aarhpgn', 'ZTF19aatjfny', 'ZTF19aasemsm', 'ZTF19aatgshx', 'ZTF19aarsyyk', 'ZTF19aaugngt', 'ZTF19aaufzcj', 'ZTF19aathabc', 'ZTF19aarioel', 'ZTF19aaujiwc', 'ZTF19aauishy', 'ZTF18aasympo', 'ZTF19aatnfyv', 'ZTF19aatnfvj', 'ZTF19aaqyovr', 'ZTF19aarycuy', 'ZTF19aaryace', 'ZTF19aauitks', 'ZTF19aauisdr', 'ZTF19aaptbsf', 'ZTF18abhjwvl', 'ZTF19aauplxw', 'ZTF19aaujlou', 'ZTF19aaupklw', 'ZTF19aauhhjy', 'ZTF19aasjatz', 'ZTF19aaujxcd', 'ZTF19aauvjws', 'ZTF19aaumrok', 'ZTF19aaupkrl', 'ZTF19aaulrch', 'ZTF18aaaqgyc', 'ZTF19aavkwcg', 'ZTF19aavkvtm', 'ZTF19aavkwmx', 'ZTF19aavkxgt', 'ZTF19aavledx', 'ZTF19aavldty', 'ZTF19aavlgdy', 'ZTF19aavhyzx', 'ZTF18aaqkcso', 'ZTF19aavjfha', 'ZTF19aavhblr', 'ZTF19aavmlqh', 'ZTF19aatubsj', 'ZTF19aavnotz', 'ZTF19aavlgjq', 'ZTF19aavocxo', 'ZTF19aavlghe', 'ZTF19aavhymd', 'ZTF19aavipfk', 'ZTF19aavoqxg', 'ZTF19aavoudg', 'ZTF18abetcyu', 'ZTF19aavlhmy', 'ZTF19aavnven', 'ZTF19aavgyjd', 'ZTF18aaqcdiv', 'ZTF19aavkuig', 'ZTF19aavnwtu', 'ZTF19aauivtj', 'ZTF19aavrsae', 'ZTF19aarnreg', 'ZTF19aavlnqp', 'ZTF19aavsahh', 'ZTF19aavrthg', 'ZTF19aavhwxs', 'ZTF19aavlnik', 'ZTF18aboczgw', 'ZTF19aavvdfi', 'ZTF19aaqtpet', 'ZTF19aavwexx', 'ZTF19aavqotz', 'ZTF19aavvzwq', 'ZTF19aavwbpc', 'ZTF19aavwcow', 'ZTF19aavweqz', 'ZTF19aavxfib', 'ZTF19aavnzti', 'ZTF18aarlukc', 'ZTF19aauvlhw', 'ZTF19aavxrzc', 'ZTF19aavlesn', 'ZTF19aavxjcr', 'ZTF19aapfoax', 'ZTF19aauqywe', 'ZTF19aawbgdh', 'ZTF19aavanad', 'ZTF19aavpzke', 'ZTF19aavaltf', 'ZTF19aawgckg', 'ZTF19aavnxwj', 'ZTF19aawgfnn', 'ZTF19aavocqa', 'ZTF19aavlfvn', 'ZTF19aavovik', 'ZTF19aavovkq', 'ZTF18aavuziy', 'ZTF19aavrxyy', 'ZTF19aavowpa', 'ZTF19aattnch', 'ZTF18aayooza', 'ZTF18accepvi', 'ZTF19aaudams', 'ZTF19aawlluc', 'ZTF19aavouav', 'ZTF19aawnguu', 'ZTF18aaqpouu', 'ZTF19aatylnl', 'ZTF19aavnwoh', 'ZTF18aahxdfs', 'ZTF19aavszux', 'ZTF19aavsqjw', 'ZTF19aavspux', 'ZTF19aawnqoj', 'ZTF19aavsahi', 'ZTF19aaqapij', 'ZTF19aawscqd', 'ZTF19aavyvkz', 'ZTF19aavnxgx', 'ZTF19aavqoop', 'ZTF19aavwdwy', 'ZTF19aavwarp', 'ZTF19aavzlin', 'ZTF19aavnurw', 'ZTF18aceqkmg', 'ZTF19aavrngz', 'ZTF19aabzzvo', 'ZTF19aapzmzm', 'ZTF19aadhdzx', 'ZTF19aavajrd', 'ZTF19aawfanq', 'ZTF19aaweqnm', 'ZTF19aawfwwv', 'ZTF19aaxbxui', 'ZTF19aawhagd', 'ZTF19aatjgrc', 'ZTF19aavwgol', 'ZTF19aawwlxc', 'ZTF19aavoayk', 'ZTF19aaxffum', 'ZTF18acugblr', 'ZTF19aavnxna', 'ZTF19aavlhjo', 'ZTF19aasbxon', 'ZTF19aawsyhf', 'ZTF19aavnnzk', 'ZTF19aawllso', 'ZTF19aaxibpz', 'ZTF19aaujlpo', 'ZTF19aavnvnx', 'ZTF19aavqiti', 'ZTF19aaxpjpq', 'ZTF19aatvmcr', 'ZTF19aaprwon', 'ZTF19aawmhbu', 'ZTF19aawyepw', 'ZTF18aasvfah', 'ZTF19aawafvn', 'ZTF19aawlgne', 'ZTF19aawupxb', 'ZTF19aavqhfi', 'ZTF19aavrnol', 'ZTF19aardooq', 'ZTF18accvhmi', 'ZTF19aawgqcc', 'ZTF19aayksry', 'ZTF19aavnxfh', 'ZTF19aawyuxv', 'ZTF19aaxdafs', 'ZTF19aauctnx', 'ZTF18acrdlrp', 'ZTF18aaikyip', 'ZTF18acqsqry', 'ZTF18acrukmr', 'ZTF18acrukrr', 'ZTF18acrvcqx', 'ZTF17aaaoqmq', 'ZTF18acrxfwp', 'ZTF18acrxiod', 'ZTF18aaacnpb', 'ZTF18aazwtfj', 'ZTF18acrtykk', 'ZTF18acsjjqr', 'ZTF18acapgdp', 'ZTF18acrfvuf', 'ZTF18acrctos', 'ZTF18acsxsmi', 'ZTF18acrciyr', 'ZTF18acrcizr', 'ZTF18acrvakb', 'ZTF17aabvtgd', 'ZTF18acrultm', 'ZTF18acsyzla', 'ZTF18acsxrge', 'ZTF18acrukpd', 'ZTF18acenyfr', 'ZTF18acrlmvt', 'ZTF18acujaic', 'ZTF18acurbia', 'ZTF18acuring', 'ZTF18acueemg', 'ZTF18acrdtzn', 'ZTF18acuqyyn', 'ZTF18acuqxyf', 'ZTF18acuqskr', 'ZTF18acuschq', 'ZTF18acszxqc', 'ZTF18acsouto', 'ZTF18aajhfdm', 'ZTF18actxhgn', 'ZTF18acrwgfp', 'ZTF18acvddao', 'ZTF18acubbwb', 'ZTF18acrvldh', 'ZTF18acurwua', 'ZTF18acrgunw', 'ZTF18aawxcbp', 'ZTF18aawmxdx', 'ZTF18acurljz', 'ZTF18acrxgbu', 'ZTF18acsowoc', 'ZTF18acueeoo', 'ZTF18acrdtwo', 'ZTF18acslbve', 'ZTF18acvvudh', 'ZTF18acvwchw', 'ZTF18acvwcky', 'ZTF18aaiuynw', 'ZTF18acvwcms', 'ZTF18acvwddd', 'ZTF18acvvved', 'ZTF18acsyanq', 'ZTF18acrexje', 'ZTF18acwhaxj', 'ZTF18acszixl', 'ZTF18acvvuzi', 'ZTF18acvwgqy', 'ZTF18acvwgdu', 'ZTF18acvwbos', 'ZTF17aaanppf', 'ZTF18acubbuv', 'ZTF18acwyvhw', 'ZTF18acuewyy', 'ZTF18aaquudq', 'ZTF18acxgujm', 'ZTF18acetnxh', 'ZTF18acszgpa', 'ZTF17aabwgbf', 'ZTF18aawonvg', 'ZTF18acynmcp', 'ZTF18acsxvpj', 'ZTF18aczawcw', 'ZTF18acxijqw', 'ZTF18actzpxw', 'ZTF18acrunrq', 'ZTF18acxyarg', 'ZTF18acvwhaz', 'ZTF18aczcsmg', 'ZTF18acvgugm', 'ZTF18acvgzab', 'ZTF18aczeraw', 'ZTF18aczerlj', 'ZTF18acuswcl', 'ZTF18acrxtrv', 'ZTF18acynlmf', 'ZTF18acwugwx', 'ZTF18acybdar', 'ZTF18acvgxmp', 'ZTF18acwtykp', 'ZTF18acvgwbt', 'ZTF18aczeqva', 'ZTF18achyngk', 'ZTF18acvdstg', 'ZTF18adaysgv', 'ZTF18acrvdsj', 'ZTF18aaacbjk', 'ZTF18acurmhj', 'ZTF18acvggao', 'ZTF18acywbgz', 'ZTF18adaylyq', 'ZTF18acyxxen', 'ZTF18adalarn', 'ZTF18adaliyg', 'ZTF18acrhegn', 'ZTF18acwzecq', 'ZTF18adalglq', 'ZTF18adaimlf', 'ZTF18adbywzj', 'ZTF18aczenvx', 'ZTF18adazbol', 'ZTF18adbifqw', 'ZTF18aabeyio', 'ZTF18aabgazt', 'ZTF18acwtyfp', 'ZTF18adbmkvr', 'ZTF18aczpdvz', 'ZTF18adcbymm', 'ZTF18adcbytv', 'ZTF18adbdhni', 'ZTF18adbdhtn', 'ZTF18acvrgwv', 'ZTF18acvqsht', 'ZTF18adcedvg', 'ZTF18adaymez', 'ZTF18aczevei', 'ZTF19aaacubf', 'ZTF19aaaegdw', 'ZTF18acuspbi', 'ZTF18adazigr', 'ZTF18adaktri', 'ZTF18acvgqsl', 'ZTF18adbabqs', 'ZTF18aawngqo', 'ZTF19aaafgdr', 'ZTF18adbabqv', 'ZTF18acsowfw', 'ZTF18acyxnyw', 'ZTF18acqssac', 'ZTF19aaafhyc', 'ZTF18aajkgtr', 'ZTF19aaafhdx', 'ZTF19aaafzah', 'ZTF18adaadmh', 'ZTF18adbmiyb', 'ZTF18adbmbgf', 'ZTF18acsxkme', 'ZTF18adbcjkb', 'ZTF18abwtbha', 'ZTF18actadvo', 'ZTF18adasjei', 'ZTF18acytbjt', 'ZTF18acxhvzb', 'ZTF18adbcjin', 'ZTF18adamhjn', 'ZTF18aczdetq', 'ZTF18acvvpov', 'ZTF18acurjjx', 'ZTF18abgkjpd', 'ZTF18acrnhtg', 'ZTF18abedvgv', 'ZTF18adbabzm', 'ZTF18adbacwh', 'ZTF18acueykv', 'ZTF18acnoktl', 'ZTF19aaapnxn', 'ZTF18acyendr', 'ZTF18aavjqvm', 'ZTF18adbadid', 'ZTF18adalfyk', 'ZTF19aaaanri', 'ZTF18abxrhmi', 'ZTF19aaahrff', 'ZTF19aaahryn', 'ZTF19aaaaxeu', 'ZTF18acvdcyf', 'ZTF18acrttfl', 'ZTF18acsybmh', 'ZTF19aaahrno', 'ZTF18actafdw', 'ZTF19aaahsjp', 'ZTF18acukgkk', 'ZTF18adbmmci', 'ZTF18aczwdth', 'ZTF19aaaokub', 'ZTF19aaafldf', 'ZTF19aaafnhx', 'ZTF19aabyyhr', 'ZTF19aaafice', 'ZTF19aaaplqp', 'ZTF19aacivvy', 'ZTF18acwzawa', 'ZTF18acvqkxx', 'ZTF18adbdhwh', 'ZTF18acsxssp', 'ZTF19aacosqh', 'ZTF18actzyzs', 'ZTF19aaabmhd', 'ZTF19aacolgi', 'ZTF18aczawvm', 'ZTF19aaaanyf', 'ZTF18acsyfnk', 'ZTF19aabgefm', 'ZTF19aacqeai', 'ZTF19aabhfdi', 'ZTF18abzbprc', 'ZTF19aaaolce', 'ZTF18acvqugw', 'ZTF18acsmisj', 'ZTF18adazgdh', 'ZTF18aarbklo', 'ZTF18aajgowk', 'ZTF18acurklz', 'ZTF19aabyvwv', 'ZTF19aaczyek', 'ZTF19aabyppp', 'ZTF19aaahjhc', 'ZTF18adbcjdl', 'ZTF18aawohdr', 'ZTF19aadfxra', 'ZTF18acuepcb', 'ZTF19aaafoaf', 'ZTF19aabzgoc', 'ZTF19aaapvoy', 'ZTF18aahegog', 'ZTF19aadoevn', 'ZTF19aacxvyw', 'ZTF19aacivld', 'ZTF19aadyiao', 'ZTF19aacwqjs', 'ZTF19aadomgl', 'ZTF19aaeoqkk', 'ZTF18aacdbzx', 'ZTF19aadgmuq', 'ZTF19aadnxkc', 'ZTF19aaapjps', 'ZTF18acqighv', 'ZTF19aaesfkm', 'ZTF19aaflhbi', 'ZTF18aaisvqg', 'ZTF18aaisjkm', 'ZTF19aaeomsn', 'ZTF19aadgceq', 'ZTF19aaeopbq', 'ZTF19aadfumv', 'ZTF19aaciwam', 'ZTF19aadohat', 'ZTF19aaejtrn', 'ZTF18abcgryu', 'ZTF19aafmidf', 'ZTF19aacsojb', 'ZTF18acvghyv', 'ZTF18aaofdgl', 'ZTF19aailmac', 'ZTF19aailsad', 'ZTF19aailvhf', 'ZTF19aailvdf', 'ZTF18acvdwhd', 'ZTF19aaeraih', 'ZTF19aagnprs', 'ZTF19aajwkbs', 'ZTF19aaaemtp', 'ZTF19aacsofi', 'ZTF19aaikxuj', 'ZTF19aailpww', 'ZTF19aailqpe', 'ZTF19aaaplph', 'ZTF19aailtfn', 'ZTF19aafmyow', 'ZTF19aailtfi', 'ZTF19aaapljd', 'ZTF18adasxhd', 'ZTF19aakljys', 'ZTF19aaeuhgo', 'ZTF19aakswrb', 'ZTF18abvzqwo', 'ZTF19aakykjl', 'ZTF19aalahqa', 'ZTF19aajykpl', 'ZTF18aceotqg', 'ZTF18abixrei', 'ZTF19aaknvlv', 'ZTF19aaknvuh', 'ZTF19aambixl', 'ZTF19aaktpns', 'ZTF19aailrlv', 'ZTF19aalubnr', 'ZTF18aavrmcg', 'ZTF18aayxupv', 'ZTF18aatybbp', 'ZTF18aaziklu', 'ZTF17aaarqox', 'ZTF18aazlsvk', 'ZTF18aajumjz', 'ZTF17aaawcwx', 'ZTF18aazwdak', 'ZTF17aabumwe', 'ZTF18abaphcx', 'ZTF17aaaedvh', 'ZTF18aagvqkp', 'ZTF18aanmmeg', 'ZTF18abbkixx', 'ZTF17aaaeuus', 'ZTF18abbodly', 'ZTF18aarltqh', 'ZTF18abaqvko', 'ZTF18abaambh', 'ZTF18abbmbys', 'ZTF18abcbwnd', 'ZTF18abajspl', 'ZTF18aazhwnh', 'ZTF18abbfwtt', 'ZTF18abcezmh', 'ZTF18abcipbf', 'ZTF18abbowqa', 'ZTF18abbuxcm', 'ZTF18abashqj', 'ZTF18abcreta', 'ZTF18abchhcn', 'ZTF18aazziao', 'ZTF18aazzfhx', 'ZTF18abcptmt', 'ZTF18abcrasg', 'ZTF18abashqr', 'ZTF18abblwsh', 'ZTF18abcdzyc', 'ZTF18abcrzfz', 'ZTF18abcvush', 'ZTF18abawrxq', 'ZTF18abccpuw', 'ZTF18abdbuty', 'ZTF18abcxzqk', 'ZTF18aaseyca', 'ZTF18aansqun', 'ZTF18aajswer', 'ZTF18abcvufc', 'ZTF18abbvbru', 'ZTF18abalzzd', 'ZTF18abditme', 'ZTF18abcyawn', 'ZTF18abguhwj', 'ZTF18abehceu', 'ZTF18abfzkno', 'ZTF18abglpdy', 'ZTF18abgisys', 'ZTF18abcbvxc', 'ZTF18abespgb', 'ZTF18abeakbs', 'ZTF18aawurud', 'ZTF18abhhnnv', 'ZTF18abcfcka', 'ZTF18abdkini', 'ZTF18abgulkc', 'ZTF18abhqdpv', 'ZTF18abetewu', 'ZTF18abaqetc', 'ZTF18abflqxq', 'ZTF18abgitic', 'ZTF17aabvfha', 'ZTF18aapaohn', 'ZTF18abkjfzt', 'ZTF18abkjfzw', 'ZTF18abkhcrj', 'ZTF18aaofsmp', 'ZTF18abhakmn', 'ZTF18abjbnes', 'ZTF18aaqkdwf', 'ZTF18abjstcm', 'ZTF18ablhplf', 'ZTF18aaoxryq', 'ZTF18abjndls', 'ZTF18abfzixe', 'ZTF18abjvgwv', 'ZTF18abdzvgz', 'ZTF18abiirfq', 'ZTF18abklbgb', 'ZTF18abeibdj', 'ZTF18ablowct', 'ZTF18abjwahi', 'ZTF18ablqfmf', 'ZTF18abeajml', 'ZTF18ablqlzp', 'ZTF18ablqqeb', 'ZTF18abkifng', 'ZTF18ablrlbm', 'ZTF18abcpdns', 'ZTF18ablsxjo', 'ZTF18abkhcax', 'ZTF18abkhbrs', 'ZTF18abcfdzu', 'ZTF18ablongw', 'ZTF18abjhbss', 'ZTF18abelhrm', 'ZTF18abeitow', 'ZTF18ablxdqz', 'ZTF18abfxhrt', 'ZTF18abgqvwv', 'ZTF18abgubgi', 'ZTF18abmjgyk', 'ZTF18abmjujg', 'ZTF18abjyjdz', 'ZTF18ablrljh', 'ZTF18abmbiza', 'ZTF18aamvand', 'ZTF18abmnwvc', 'ZTF18ablwbqn', 'ZTF17aaartnt', 'ZTF18ablszje', 'ZTF18aabesgz', 'ZTF18abmxfrc', 'ZTF18abmmtts', 'ZTF17aabvcvr', 'ZTF18ablzvih', 'ZTF18ablsypo', 'ZTF18abbydbi', 'ZTF18abmrhom', 'ZTF18abltbrd', 'ZTF18aanxtko', 'ZTF18abnnnzk', 'ZTF18abncizy', 'ZTF18abmdvcj', 'ZTF18abctyvq', 'ZTF18ablvjom', 'ZTF18abnchro', 'ZTF18aabqtsw', 'ZTF18aboemit', 'ZTF18abnugci', 'ZTF18abntxck', 'ZTF18abmogca', 'ZTF18abmwxvv', 'ZTF18abobkii', 'ZTF18abiitmq', 'ZTF18abahgfm', 'ZTF18aboaeqy', 'ZTF18abfgabf', 'ZTF18abflcof', 'ZTF18abgugua', 'ZTF18abjgzxw', 'ZTF18abhozbd', 'ZTF18abeaymc', 'ZTF18abnuhuy', 'ZTF18abptsco', 'ZTF18abpttky', 'ZTF18abqjvyl', 'ZTF18abmkbqk', 'ZTF18abrzhik', 'ZTF18abnbhpu', 'ZTF18abccnyj', 'ZTF18abcctas', 'ZTF17aaazypn', 'ZTF18abncimo', 'ZTF18aabfmux', 'ZTF18absdeij', 'ZTF18abmwzpt', 'ZTF18aaymqbw', 'ZTF18ablmhjv', 'ZTF18abrlurr', 'ZTF18aarrelk', 'ZTF18ablrlmu', 'ZTF18abimjrs', 'ZTF18aamfybu', 'ZTF18abctdgj', 'ZTF18abokvkt', 'ZTF18abnygkb', 'ZTF18abmnvjc', 'ZTF18abrymgj', 'ZTF18aaakpjg', 'ZTF18abmddcb', 'ZTF18ablwzih', 'ZTF18abnvnqb', 'ZTF18abeachz', 'ZTF18abkvwzj', 'ZTF18aapgbfm', 'ZTF18ablpnuq', 'ZTF18abihxip', 'ZTF18aauherh', 'ZTF18abdlfev', 'ZTF18absyyig', 'ZTF18abdazfd', 'ZTF18absgnio', 'ZTF18abtdxke', 'ZTF18abteera', 'ZTF18abotcjv', 'ZTF18abotbuq', 'ZTF18abhnscs', 'ZTF18abtjlsr', 'ZTF18abtjmns', 'ZTF18abnufuv', 'ZTF18abjmlty', 'ZTF18abbuxbk', 'ZTF18abryqnn', 'ZTF17aabvele', 'ZTF18absljwl', 'ZTF18abtnfnq', 'ZTF18abhowyg', 'ZTF18abscghc', 'ZTF18absqitc', 'ZTF18absqkfg', 'ZTF18abscyhs', 'ZTF18abnvklh', 'ZTF18abmqwgr', 'ZTF18abqblsk', 'ZTF18abshnwm', 'ZTF18abrzbnb', 'ZTF18abrzcdi', 'ZTF18abrqedj', 'ZTF18absrcps', 'ZTF18abslxpd', 'ZTF18abcwjii', 'ZTF18abuatdz', 'ZTF18abjlxwv', 'ZTF18abufdlx', 'ZTF18abskhcp', 'ZTF18abtsobl', 'ZTF18abtmbaz', 'ZTF18abtswjk', 'ZTF18abrzbuj', 'ZTF18aaabzcd', 'ZTF18absmqmw', 'ZTF18abeyoaa', 'ZTF18abdmghg', 'ZTF18abcwzhs', 'ZTF18abgxnrw', 'ZTF18abcgjpn', 'ZTF18abnwvmh', 'ZTF18abijktl', 'ZTF18abhpyor', 'ZTF18aborolm', 'ZTF18abswypg', 'ZTF18abttaxw', 'ZTF18absbyio', 'ZTF18abotfiv', 'ZTF18abbqjbb', 'ZTF18absnayk', 'ZTF18abtnwpa', 'ZTF18abuqugw', 'ZTF18abthuby', 'ZTF18abukavn', 'ZTF18abtvinn', 'ZTF18abuwxii', 'ZTF18abnzzvq', 'ZTF18abuatqi', 'ZTF18abnygwk', 'ZTF18abbuutk', 'ZTF18abufgdj', 'ZTF18abtswll', 'ZTF18abuhyjv', 'ZTF18abvgkcg', 'ZTF18aboztku', 'ZTF18aajsypz', 'ZTF18absnqyo', 'ZTF18abvivzm', 'ZTF18abugmrg', 'ZTF18abtphnh', 'ZTF17aaafrch', 'ZTF18abuhzfc', 'ZTF18abtfwhe', 'ZTF18abmjuya', 'ZTF18abeainm', 'ZTF18abtxynh', 'ZTF18abvfisg', 'ZTF18abvujhp', 'ZTF18abbmenz', 'ZTF18abdebec', 'ZTF18abvwsjn', 'ZTF18abtjrbt', 'ZTF18abskrcm', 'ZTF18aajclem', 'ZTF18abumdjo', 'ZTF18abnzksu', 'ZTF18abvfafa', 'ZTF18abwgaek', 'ZTF18abvzdvj', 'ZTF18abwkans', 'ZTF18abuqhje', 'ZTF18abusyhc', 'ZTF17aabptoh', 'ZTF18absggvu', 'ZTF18abuuqkb', 'ZTF18abuvrnw', 'ZTF18abuxjrd', 'ZTF18abunhyu', 'ZTF18abwkhky', 'ZTF18abuahnw', 'ZTF18abtzzlv', 'ZTF18abvcehx', 'ZTF18abvptts', 'ZTF18abvtbow', 'ZTF18abtnggb', 'ZTF18aazttzj', 'ZTF18abtogdl', 'ZTF18ablprcf', 'ZTF18aburkuo', 'ZTF18abxdtyz', 'ZTF18abehvba', 'ZTF18abslbyb', 'ZTF18abwkrvs', 'ZTF18abmwwni', 'ZTF17aacjaxh', 'ZTF18abvpnnb', 'ZTF18abqwixr', 'ZTF18abvrvdk', 'ZTF18abqaujs', 'ZTF17aabvwlq', 'ZTF18abltbxa', 'ZTF18abxyvxq', 'ZTF18abtlwoj', 'ZTF18abwfktb', 'ZTF18abuiytq', 'ZTF18abqyvzy', 'ZTF18aabgboh', 'ZTF17aabsopd', 'ZTF17aaatdgc', 'ZTF17aaaabds', 'ZTF18abijaxd', 'ZTF18aaeyilr', 'ZTF18abuqoii', 'ZTF18abrwpvo', 'ZTF18abwlupf', 'ZTF18abolvzu', 'ZTF18absnxdb', 'ZTF17aaaeizh', 'ZTF18abvefxs', 'ZTF18abyfcns', 'ZTF18abyfvfr', 'ZTF18abchrps', 'ZTF18abufhtw', 'ZTF18acafztq', 'ZTF18abykeda', 'ZTF18abuifwq', 'ZTF18abtrlla', 'ZTF17aacxpvk', 'ZTF18acaksoq', 'ZTF18abxsmlf', 'ZTF18ablvxdg', 'ZTF18acaubtl', 'ZTF17aaaadiz', 'ZTF18acbvkwl', 'ZTF18acbvifx', 'ZTF18aadrhsi', 'ZTF18acbvzta', 'ZTF18acbwdwg', 'ZTF18abuszlm', 'ZTF18abmfwbn', 'ZTF18acbwhns', 'ZTF18acbwppb', 'ZTF18acbwwpm', 'ZTF18acbwxcc', 'ZTF18acbwxou', 'ZTF18abmefij', 'ZTF18acbxhua', 'ZTF18acbxkop', 'ZTF18abviyvv', 'ZTF18acbxsge', 'ZTF18abtlxdk', 'ZTF18abxmbjy', 'ZTF18abtbxou', 'ZTF18acbvmwr', 'ZTF17aadkbbu', 'ZTF18aaadyuc', 'ZTF18acbzqyi', 'ZTF18aaasgau', 'ZTF18abyfbhl', 'ZTF18acbwfvt', 'ZTF18aaemivw', 'ZTF18abwwdaz', 'ZTF18accdsxf', 'ZTF18accfxnn', 'ZTF18abtpite', 'ZTF18abmenfr', 'ZTF18acchzyo', 'ZTF18abvmkfh', 'ZTF18acckpro', 'ZTF18acbvbox', 'ZTF18abtswhs', 'ZTF18acbuibx', 'ZTF18acbvijf', 'ZTF18acclexy', 'ZTF18accpjff', 'ZTF18acbwopo', 'ZTF18acauwik', 'ZTF18acbyyff', 'ZTF18abwjzxr', 'ZTF18abmolpg', 'ZTF18acbvmzg', 'ZTF18aadkjmo', 'ZTF18abztrfp', 'ZTF18acbzpmu', 'ZTF18aaqkjuw', 'ZTF18aakeywu', 'ZTF18acbznkf', 'ZTF18aalpega', 'ZTF18acbzojv', 'ZTF18acbzpgy', 'ZTF18acdvvhi', 'ZTF18acdways', 'ZTF18abasoah', 'ZTF18abzscns', 'ZTF18absqpbs', 'ZTF18accjecm', 'ZTF18acchhpe', 'ZTF18accicnc', 'ZTF18accjuwd', 'ZTF18acectzf', 'ZTF18accnnyu', 'ZTF18accnbgw', 'ZTF18acbznpp', 'ZTF18acbwfim', 'ZTF18abkwfxq', 'ZTF18abujubn', 'ZTF18aceijsp', 'ZTF18abgjezf', 'ZTF18acbxoie', 'ZTF18absgmhg', 'ZTF18acchlkx', 'ZTF18accvkpt', 'ZTF18accwihs', 'ZTF18acbzoxp', 'ZTF18accfjmp', 'ZTF18abudvzn', 'ZTF18abtjmfm', 'ZTF18abcplgw', 'ZTF17aaakeap', 'ZTF18accvflv', 'ZTF18aafxtrt', 'ZTF18accnlji', 'ZTF18accdtch', 'ZTF18abxjekh', 'ZTF18aceyzwi', 'ZTF18acfejol', 'ZTF18abxzsnv', 'ZTF17aaaruzy', 'ZTF18aaagyuv', 'ZTF18aaapvkn', 'ZTF18accpjkb', 'ZTF18abmrhkl', 'ZTF18acbwtng', 'ZTF18abupgps', 'ZTF18achavgj', 'ZTF18acbvmjq', 'ZTF17aacxkaq', 'ZTF18aceolcv', 'ZTF18acbzvqn', 'ZTF18aamsgjq', 'ZTF18acbzwna', 'ZTF18acbxnpg', 'ZTF18abqavrw', 'ZTF18abnczui', 'ZTF18acfwmqj', 'ZTF18abnzunr', 'ZTF17aacfeuw', 'ZTF17aaayvyb', 'ZTF18acmrkpm', 'ZTF18accsane', 'ZTF18acchhxu', 'ZTF18abtyrmh', 'ZTF18abcpddh', 'ZTF18abnzuqm', 'ZTF18aceaape', 'ZTF18acbudwt', 'ZTF18acdvahc', 'ZTF18acewwqf', 'ZTF18acbwdxy', 'ZTF18abttklf', 'ZTF18aavrleg', 'ZTF18acegbeo', 'ZTF18abxbnvq', 'ZTF18aawmppg', 'ZTF18acnbptb', 'ZTF18aamrkmd', 'ZTF18achdnip', 'ZTF18achdidy', 'ZTF18achdfqm', 'ZTF18acnngyx', 'ZTF18acpeesd', 'ZTF18aceyqpm', 'ZTF18acnbglg', 'ZTF18acdvqhn', 'ZTF18aceykmc', 'ZTF18acbzvzm', 'ZTF18acnbejq', 'ZTF18aathofv', 'ZTF18acppzle', 'ZTF18ackvfyc', 'ZTF18abbuzeu', 'ZTF18abnuwsj', 'ZTF18aaadsrj', 'ZTF18acefgee', 'ZTF18accvmgs', 'ZTF18acnbgpw', 'ZTF18aadezmv', 'ZTF18acnbgvd', 'ZTF18aabklju', 'ZTF18acpuici', 'ZTF18abwskcn', 'ZTF18cnnfup')
    # nuclear_transients_080819 = np.unique(('ZTF19aarioci', 'ZTF19aabbnzo', 'ZTF18abxftqm', 'ZTF19aapreis', 'ZTF19abfdupx', 'ZTF19abfjjwc', 'ZTF19abfjjwc', 'ZTF19abflufo', 'ZTF19abfdupx', 'ZTF19abfqmqz', 'ZTF19abfrydu', 'ZTF19abfvhlx', 'ZTF19abgbdcp', 'ZTF19abgbdcp', 'ZTF19abfdupx', 'ZTF19abflufo', 'ZTF19abfdupx', 'ZTF19abfqmqz', 'ZTF19abfqmqz', 'ZTF19abezcns', 'ZTF19abggmrt', 'ZTF19abgbtla', 'ZTF19abgcbey', 'ZTF19abfvhlx', 'ZTF19abgjlef', 'ZTF19abfvhlx', 'ZTF19abfwfiq', 'ZTF19abfwhja', 'ZTF19abglmpf', 'ZTF19abgbdcp', 'ZTF19abglmpf', 'ZTF19abfjjwc', 'ZTF19abgncfz', 'ZTF19abgffaj', 'ZTF19abipkyb', 'ZTF19abfrydu', 'ZTF19abgcbey', 'ZTF19abfrydu', 'ZTF19abgpnge', 'ZTF19abgncfz', 'ZTF19abgbtla', 'ZTF19abgcbey', 'ZTF19abglmpf', 'ZTF19abfvhlx', 'ZTF19abgjlef', 'ZTF18abtlqoh', 'ZTF19abfvfxp', 'ZTF19abgrhzk', 'ZTF18aarwxum', 'ZTF19abglmpf', 'ZTF19abgrhzk', 'ZTF19abfwfiq', 'ZTF19abgdcyx', 'ZTF19abglmpf', 'ZTF19abgrcxs', 'ZTF19abgmkef', 'ZTF19abfzzxy', 'ZTF19abgbdcp', 'ZTF19abfjjwc', 'ZTF19abgncfz', 'ZTF19abgbtla', 'ZTF19abfdupx', 'ZTF19abglmpf', 'ZTF19aambfxc', 'ZTF19abfdupx', 'ZTF19abeeqoj', 'ZTF19aambfxc', 'ZTF19abfdupx', 'ZTF19abgctni', 'ZTF19abgjfoj', 'ZTF19abhbkdd', 'ZTF19abgpnge', 'ZTF19abgcbey', 'ZTF19abgppki', 'ZTF19abgpnge', 'ZTF19abfrydu', 'ZTF19abgppki', 'ZTF19abgpjed', 'ZTF19abgbtla', 'ZTF19abglmpf', 'ZTF19abflufo', 'ZTF19abghldi', 'ZTF18aarwxum', 'ZTF19abglmpf', 'ZTF19abgqksj', 'ZTF19abgpjed', 'ZTF19abheryh', 'ZTF19abgbtla', 'ZTF19abfdupx', 'ZTF19abghldi', 'ZTF19abglmpf', 'ZTF19abgdcyx', 'ZTF19abghldi', 'ZTF19abglmpf', 'ZTF19abgmkef', 'ZTF19abgrcxs', 'ZTF19abfwhja', 'ZTF19abgbdcp', 'ZTF19abfjjwc', 'ZTF19abgffaj', 'ZTF19abgjfoj', 'ZTF19abfqmqz', 'ZTF19abfvhlx', 'ZTF19abhzdjp', 'ZTF19abiaxpi', 'ZTF19abgvfst', 'ZTF19abglmpf', 'ZTF18aarwxum', 'ZTF19abiiqcu', 'ZTF18aarwxum', 'ZTF19abgjlef', 'ZTF19abicwzc', 'ZTF19abgjlef', 'ZTF19abgqksj', 'ZTF19abhjhes', 'ZTF19abfjjwc', 'ZTF19abhceez', 'ZTF19abheryh', 'ZTF19abgrhzk', 'ZTF19abhenjb', 'ZTF19abgdcyx', 'ZTF19abgrhzk', 'ZTF19abhenjb', 'ZTF19abgbdcp', 'ZTF18aamasph', 'ZTF19abgncfz', 'ZTF19abgdcyx', 'ZTF19abglmpf', 'ZTF19abgpydp', 'ZTF19abglmpf', 'ZTF19abghldi', 'ZTF19abgpydp', 'ZTF19abgctni', 'ZTF19abidfag', 'ZTF19abipnwp', 'ZTF19abfvhlx', 'ZTF19abipktm', 'ZTF19abfqmqz', 'ZTF19abgfnmt', 'ZTF19abipnwp', 'ZTF19abfzwpe', 'ZTF19abiptub', 'ZTF19abfvhlx', 'ZTF19abgfnmt', 'ZTF19abiqpux', 'ZTF19abiszzn', 'ZTF19abgpjed', 'ZTF19abhzdjp', 'ZTF19abgdcyx', 'ZTF19aambfxc', 'ZTF19abgcbey', 'ZTF19abghldi', 'ZTF19abgvfst', 'ZTF19abhzdjp', 'ZTF19abfrydu', 'ZTF19abgncfz', 'ZTF19abiszzn', 'ZTF19abgpjed', 'ZTF19abggmrt', 'ZTF19abgppki', 'ZTF19abiietd', 'ZTF19abgcbey', 'ZTF19abglmpf', 'ZTF19abgqksj', 'ZTF19abgjlef', 'ZTF19abgjoth', 'ZTF19abidbya', 'ZTF19abgmkef', 'ZTF19abhjhes', 'ZTF19abgrcxs', 'ZTF19abgqksj', 'ZTF19abgqksj', 'ZTF19abgmkef', 'ZTF19abhjhes', 'ZTF19abgjlef', 'ZTF19abgrcxs', 'ZTF19abfwhja', 'ZTF19abgncfz', 'ZTF19abfjjwc', 'ZTF19abgvfst', 'ZTF19abglmpf', 'ZTF19abgvfst', 'ZTF19abfdupx', 'ZTF19abjgbgc', 'ZTF19abgmjtu', 'ZTF19abjgdko', 'ZTF19abicvxs', 'ZTF19aambfxc', 'ZTF18aarwxum', 'ZTF18aarwxum', 'ZTF19abglmpf', 'ZTF19abiijov', 'ZTF18aarwxum', 'ZTF19abfdupx', 'ZTF18aarwxum', 'ZTF19abglmpf', 'ZTF19abgpydp', 'ZTF19abgctni', 'ZTF19abipkyb', 'ZTF19abgjfoj', 'ZTF19abgffaj', 'ZTF19abiptub', 'ZTF19abfzwpe', 'ZTF19abjnzsi', 'ZTF19abiovhj', 'ZTF19abhdxcs', 'ZTF19abgjfoj', 'ZTF19abidfag', 'ZTF19abgmmfu', 'ZTF19abipmfl', 'ZTF19abisbgx', 'ZTF19abgrcxs', 'ZTF19aaqfrrl', 'ZTF19abfzzxy', 'ZTF19abhdlxp', 'ZTF19abhdvme', 'ZTF19abhusrq', 'ZTF19abisbgx', 'ZTF19abhdlxp', 'ZTF19abjravi', 'ZTF19abiietd', 'ZTF19abhenjb', 'ZTF19abgbdcp', 'ZTF19abglmpf', 'ZTF19abfdupx', 'ZTF18aarwxum', 'ZTF19abgppki', 'ZTF19abgpnge', 'ZTF19abiietd', 'ZTF19abgpjed', 'ZTF19abgbdcp', 'ZTF19abgncfz', 'ZTF19abhzdjp', 'ZTF19abjibet', 'ZTF19abiszzn', 'ZTF19abgppki', 'ZTF19abhoyxd', 'ZTF19abiaxpi', 'ZTF19abfvhlx', 'ZTF19abidbya', 'ZTF19abiptrq', 'ZTF19abgjlef', 'ZTF19abgqksj', 'ZTF19abgmkef', 'ZTF19abhjhes', 'ZTF19abidbya', 'ZTF19abfvhlx', 'ZTF19abiovhj', 'ZTF19abipktm', 'ZTF19abkcbri', 'ZTF19abgjlef', 'ZTF19abkfmjp', 'ZTF19abkfxfb', 'ZTF19abhdxcs', 'ZTF19abgrcxs', 'ZTF19abgafkt', 'ZTF19abgmkef', 'ZTF19abhjhes', 'ZTF19abkfxfb', 'ZTF19abjgdko', 'ZTF19abhdxcs', 'ZTF18aarwxum', 'ZTF18aarwxum', 'ZTF19abkcbri', 'ZTF19ablesob', 'ZTF19abidbya', 'ZTF19abgmmfu', 'ZTF19abipmfl', 'ZTF19abjnzsi', 'ZTF19abiqqve', 'ZTF19abgmjtu', 'ZTF19abicvxs', 'ZTF19abjgbwx', 'ZTF19abjgazt', 'ZTF19abjgdxx', 'ZTF19abjgdko', 'ZTF19ablesob', 'ZTF19abgdcyx', 'ZTF19abjibet', 'ZTF19abgvfst', 'ZTF19abghldi', 'ZTF18aarwxum', 'ZTF19abjibet', 'ZTF19abfdupx', 'ZTF19abghldi', 'ZTF18aarwxum', 'ZTF19abglmpf', 'ZTF19abgctni', 'ZTF19abgpydp', 'ZTF19abgmmfu', 'ZTF19abipmfl', 'ZTF19abidfag', 'ZTF19abjnzsi', 'ZTF19abiovhj', 'ZTF19abiovhj', 'ZTF19abiopky', 'ZTF19abgmmfu', 'ZTF19abgpydp', 'ZTF19abiptub', 'ZTF19abhusrq', 'ZTF19abgrcxs', 'ZTF19abisbgx', 'ZTF19abhdvme', 'ZTF19abhdvme', 'ZTF19abjravi', 'ZTF19abhdlxp', 'ZTF19abhdvme', 'ZTF19abhusrq', 'ZTF19abirbnk', 'ZTF19abiqpux', 'ZTF19ablovot', 'ZTF19abjpick', 'ZTF19aaqfrrl', 'ZTF19abgppki', 'ZTF19abhbtlo', 'ZTF18aawfquu', 'ZTF19abhzdjp', 'ZTF19abjioie', 'ZTF19abgcbey', 'ZTF19abhzdjp', 'ZTF19abiszzn', 'ZTF19abgppki', 'ZTF19abiietd', 'ZTF19abfdupx', 'ZTF18aarwxum', 'ZTF19abglmpf', 'ZTF19abjibet', 'ZTF18aarwxum', 'ZTF19abfdupx', 'ZTF18aarwxum', 'ZTF19abglmpf', 'ZTF19abfvhlx', 'ZTF19abipktm', 'ZTF19abfvhlx', 'ZTF19abgqksj', 'ZTF19abiyyun', 'ZTF19abidbya', 'ZTF19abgjlef', 'ZTF19abgmkef', 'ZTF19abhjhes', 'ZTF19abiztbh', 'ZTF19abgrcxs', 'ZTF19ablesob', 'ZTF19abgjlef', 'ZTF19abgmkef', 'ZTF19abhjhes', 'ZTF19abkdeae', 'ZTF19ablesob', 'ZTF19abgrcxs', 'ZTF19abhjhes', 'ZTF19abjpimw', 'ZTF19ablesob', 'ZTF19abjgaye', 'ZTF19abjgbgc', 'ZTF19abjgcad', 'ZTF19abgmjtu', 'ZTF19abjgdko', 'ZTF19abkfxfb', 'ZTF19abkfmjp', 'ZTF19abgmjtu', 'ZTF19abkfxfb', 'ZTF19abjgdko', 'ZTF19abgncfz', 'ZTF19abjibet', 'ZTF18aarwxum', 'ZTF19abglmpf', 'ZTF19abgdcyx', 'ZTF19abjibet', 'ZTF18aarwxum', 'ZTF18aarwxum', 'ZTF19abglmpf', 'ZTF19abjmnlw', 'ZTF19abirbnk', 'ZTF19abhusrq', 'ZTF19abhdxcs', 'ZTF19abhusrq', 'ZTF19abjpord', 'ZTF19abiqpux', 'ZTF19ablovot', 'ZTF19abjqytt', 'ZTF19abisbgx', 'ZTF19ablovot', 'ZTF19abgrcxs', 'ZTF19aaqfrrl', 'ZTF19abhoyxd', 'ZTF19abgppki', 'ZTF19abiszzn', 'ZTF19abfrydu', 'ZTF19abhzdjp', 'ZTF19abjioie', 'ZTF19abgcbey', 'ZTF19abjioie', 'ZTF19abhzdjp', 'ZTF19abgncfz', 'ZTF19abgdcyx', 'ZTF19abjibet', 'ZTF18aarwxum', 'ZTF18aarwxum', 'ZTF19abglmpf', 'ZTF19abjibet', 'ZTF18aarwxum', 'ZTF19abljkea', 'ZTF18aarwxum', 'ZTF19abglmpf', 'ZTF19abljinr', 'ZTF19abfvhlx', 'ZTF19abipktm', 'ZTF19ablesob', 'ZTF19abgjlef', 'ZTF19abgmkef', 'ZTF19abhjhes', 'ZTF19abkdlkl', 'ZTF19abgjlef', 'ZTF19abgmkef', 'ZTF19abhjhes', 'ZTF19abkdeae', 'ZTF19ablesob', 'ZTF19abhjhes', 'ZTF19abjibet', 'ZTF18aarwxum', 'ZTF19abglmpf', 'ZTF19abgdcyx', 'ZTF19abjibet', 'ZTF19abglmpf', 'ZTF19abixauz', 'ZTF19abipnwp', 'ZTF19abkevcb', 'ZTF19abipkyb', 'ZTF19abixauz', 'ZTF19abkfmjp', 'ZTF19abiqifg', 'ZTF19abkdeae', 'ZTF19ablesob', 'ZTF19ablybjv', 'ZTF19abgdcyx', 'ZTF19abjibet', 'ZTF18aarwxum', 'ZTF19abljkea', 'ZTF19abglmpf', 'ZTF19abgdcyx', 'ZTF19abjibet', 'ZTF18aarwxum', 'ZTF19abglmpf', 'ZTF19abljinr', 'ZTF19abgctni', 'ZTF19abgcbey', 'ZTF19abjibet', 'ZTF19abgvfst', 'ZTF18aarwxum', 'ZTF19abghldi', 'ZTF19abglmpf'))
    # nuclear_transients_210819 = np.unique(('ZTF19aapreis', 'ZTF18abxftqm', "ZTF18abccpuw", "ZTF19abglmpf", "ZTF19ablovot", "ZTF18acvqugw", "ZTF18acrttfl", "ZTF18acvqugw", "ZTF18acrttfl", "ZTF18acvqugw", "ZTF18acrttfl", "ZTF18acvqugw", "ZTF18acrttfl", "ZTF18acvqugw", "ZTF18acrttfl", "ZTF18acvqugw", "ZTF18acrttfl", "ZTF18acvqugw", "ZTF18acrttfl", "ZTF18acvqugw", "ZTF18acrttfl", "ZTF18acvqugw", "ZTF18acrttfl", "ZTF18acvqugw", "ZTF18acrttfl", "ZTF18acvqugw", "ZTF18acrttfl", "ZTF18acvqugw", "ZTF18acrttfl", "ZTF18acvqugw", "ZTF18acrttfl", "ZTF18acvqugw", "ZTF18acrttfl", "ZTF18acvqugw", "ZTF18acrttfl", "ZTF18acvqugw", "ZTF18acrttfl", "ZTF18acvqugw", "ZTF18acrttfl", "ZTF18acvqugw", "ZTF18acrttfl", "ZTF18acvqugw", "ZTF18acrttfl", "ZTF18acvqugw", "ZTF18acrttfl", "ZTF19abjmnlw", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aabfjyo", "ZTF18aanxnrz", "ZTF18aanxnrz", "ZTF18aaojsvk", "ZTF18abvbcln", "ZTF19ablovot", "ZTF19ablovot", "ZTF19ablovot", "ZTF19ablovot", "ZTF19ablovot", "ZTF18abmogca",))
    # nuclear_transients_210819 = np.unique(np.concatenate((nuclear_transients_080819, nuclear_transients_210819)))
    # classify_lasair_light_curves(object_names=nuclear_transients_210819, figdir='astrorapid/nuclear_transients_210819')

    # nuc_tran_plot = np.load('plot_nuclear_transients_210819.npz')
    # real_tde_probs = nuc_tran_plot['real_tde_probs']
    # probabilities = nuc_tran_plot['probabilities']
    # names = nuc_tran_plot['names']
    # radecs = nuc_tran_plot['radec']
    # peakmags = nuc_tran_plot['peakmags']
    # objids = nuc_tran_plot['objids']
    #
    # # classify_lasair_light_curves(object_names=objids[probabilities > 0.4486], figdir='astrorapid/nuclear_transients_210819')
    # np.savetxt('/Users/danmuth/PycharmProjects/astrorapid/paper_plots/TDE_candidates_210819.csv', np.hstack([radecs, peakmags, np.array([probabilities, objids]).T])[probabilities > 0.4486], fmt='%s')
    #
    # import matplotlib.pyplot as plt
    # plt.rcParams['xtick.labelsize'] = 15
    # plt.rcParams['ytick.labelsize'] = 15
    #
    # plt.figure()
    # for i in range(len(real_tde_probs)):
    #     plt.text(real_tde_probs[i]-0.025, 64, names[i], rotation=90, color='red')
    #     plt.axvline(x=real_tde_probs[i], color='red', linewidth=0.7)
    #
    # import seaborn as sns
    # sns.set_style("white")
    # print(probabilities)
    # sns.distplot(probabilities, color='dodgerblue', label='Nuclear Transients in past 30 days (from 21-Aug-2019)', kde=False, norm_hist=False)
    # plt.xlim(0, 1.)
    #
    # plt.ylabel("Number of transients", fontsize=15)
    # plt.xlabel("TDE Classification probability", fontsize=15)
    # plt.ylim(0, 250)
    # plt.legend()
    # plt.tight_layout()
    # plt.savefig('astrorapid/prob2_unnormalised_210819.pdf')
    #
    # plt.figure()
    # for i in range(len(real_tde_probs)):
    #     plt.text(real_tde_probs[i]-0.02, 180, names[i], rotation=90, color='red')
    #     plt.axvline(x=real_tde_probs[i], color='red', linewidth=0.7)
    #
    # plt.hist(probabilities, 100, cumulative=-1, color='dodgerblue', histtype='step', linewidth=1.5)
    # plt.xlim(0.3, 1.0)
    # plt.ylim(0, 200)
    # plt.xlabel("TDE Classification probability, $p$", fontsize=15)
    # plt.ylabel("Number of objects with at least $p$ probability", fontsize=15)
    # plt.tight_layout()
    # plt.savefig('astrorapid/cumulativeTDE_210819.pdf')

    # Type Ia SNe in ZTF from OSC
    snia_names = []
    import json
    with open('ZTF_snia_osc_12092019.json') as json_file:
        data = json.load(json_file)
    for i, sn in enumerate(data):
        try:
            snia_name = [name for name in data[i]['Name'].split() if 'ZTF' in name][0]
            snia_z = float(data[i]['z'].split()[0])
            snia_names.append((snia_name, snia_z))
            print(i, snia_name, snia_z)
        except IndexError as e:
            print(f"failed on {i} {data[i]['Name']}")

    classify_lasair_light_curves(object_names=snia_names, figdir='astrorapid/real_snia_from_osc/new_nodc_wredshift')




#LASAIR QUERY for nuclear transients

#SELECT DISTINCT o.objectId FROM objects o, candidates c WHERE o.sherlock_classification IN ('NT') AND o.jdmin > JDNOW() - 30 AND o.ncand > 3 AND c.objectId = o.objectId AND (c.jd > JDNOW() - 30) AND c.magpsf < 20 AND c.rb >= 0.75 AND c.nbad = 0 AND c.isdiffpos = 't' AND c.fwhm <= 5 AND ABS(c.magdiff) <= 0.1 AND c.elong <= 1.2

#SELECT DISTINCT o.objectId FROM objects o, candidates c WHERE o.sherlock_classification IN ('NT') AND o.jdmin > JDNOW() - 200 AND o.jdmin <= JDNOW() - 100 AND o.ncand > 3 AND c.objectId = o.objectId AND (c.jd > JDNOW() - 200 AND c.jd <= JDNOW() - 100) AND c.magpsf < 20 AND c.rb >= 0.75 AND c.nbad = 0 AND c.isdiffpos = 't' AND c.fwhm <= 5 AND ABS(c.magdiff) <= 0.1 AND c.elong <= 1.2

#SELECT
#objects.objectId
#FROM
#objects, candidates, sherlock_classifications
#WHERE ORDER
# sherlock_classifications.classification IN ("NT")
#         AND objects.ncand > 3
#         AND candidates.objectId = objects.objectId
#         AND candidates.magpsf < 20
#         AND candidates.rb >= 0.75
#         AND candidates.nbad = 0
#         AND candidates.isdiffpos = "t"
#         AND candidates.fwhm <= 5
#         AND ABS(candidates.magdiff) <= 0.1
#         AND candidates.elong <= 1.2
#         AND objects.jdmin > JDNOW() - 30
#         AND candidates.jd > JDNOW() - 30

#WHERE o.sherlock_classification IN ('NT')
# AND o.jdmin > JDNOW() - 200 AND o.jdmin <= JDNOW() - 100
# AND o.ncand > 3
# AND c.objectId = o.objectId
# AND (c.jd > JDNOW() - 200 AND c.jd <= JDNOW() - 100)
# AND c.magpsf < 20
# AND c.rb >= 0.75
# AND c.nbad = 0
# AND c.isdiffpos = 't'
# AND c.fwhm <= 5
# AND ABS(c.magdiff) <= 0.1
# AND c.elong <= 1.2

#WHERE o.sherlock_classification IN ('NT')
# AND o.jdmin > JDNOW() - 30
# AND o.ncand > 3
# AND c.objectId = o.objectId
# AND (c.jd > JDNOW() - 30)
# AND c.magpsf < 20
# AND c.rb >= 0.75 AND c.nbad = 0 AND c.isdiffpos = 't' AND c.fwhm <= 5 AND ABS(c.magdiff) <= 0.1 AND c.elong <= 1.2





